#include "BuilderMethods.h"
#include "GraphicObjectClass.h"
#include <string.h>
#include <ctype.h>
#include "AmigaMem.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif

void *New( PObject *self )
{
   void *new_whatever = NULL;
   struct GraphicObjectClass *class;
   struct BuilderMethods *bm;

   if( self )
   {
      if( class = (struct GraphicObjectClass *) self->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->New )
            {
               new_whatever = (*bm->New)( self );
            }
            else
               printf( "ERROR: no 'New' method for class \"%s\"\n",
                  class->ClassName );
         }
      }
   }
   return new_whatever;
}

void *NewFromClass( struct GraphicObjectClass *class )
{
   void *new_whatever = NULL;
   struct BuilderMethods *bm;

   if( bm = class->BuilderMethods )
   {
      if( bm->New )
      {
         new_whatever = (*bm->New)( NULL );
      }
      else
         printf( "ERROR: no 'New' method for class \"%s\"\n",
            class->ClassName );
   }

   return new_whatever;
}


void  Free( PObject *self )
{
   struct GraphicObjectClass *class, *selfclass;
   struct BuilderMethods *bm;

   if( self )
   {
      for( selfclass = class = (struct GraphicObjectClass *)self->isa;
           class != NULL;
           class = class->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->Free )
            {
               (*bm->Free)( self );
               return;
            }

         }
      }
      printf( "Error: %s doesn't support Free()\n", selfclass->ClassName );
   }
}


long WriteIFF
      (  GraphicObject        *self,
         FILE                 *file )
{
   struct GraphicObjectClass *class;
   struct BuilderMethods *bm;

   if( self )
   {
      if( class = (struct GraphicObjectClass *) self->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->WriteIFF )
            {
               return (*bm->WriteIFF)( self, file );
            }

         }
         printf( "Error: %s doesn't support WriteIFF()\n", class->ClassName );
      }
   }
   return -1;
}


GraphicObject* ReadIFF
      (  PClass             *class,
         FILE              *file,
         struct ChunkInfo  *chunk )
{
   struct GraphicObjectClass *gclass;
   struct BuilderMethods *bm;

   if( class )
   {
      gclass = (struct GraphicObjectClass *) class;

      if( bm = gclass->BuilderMethods )
      {
         if( bm->ReadIFF )
         {
            return (*bm->ReadIFF)( gclass, file, chunk );
         }

      }
      printf( "Error: %s doesn't support ReadIFF()\n", gclass->ClassName );
   }
   return NULL;
}



void PropEdit( GraphicObject     *self,
               struct pcgWindow  *window,
               pcg_3DPens         pens )
{
   struct GraphicObjectClass *class;
   BuilderMethods *bm;

   if( class = (struct GraphicObjectClass *) self->isa )
   {
      if( bm = class->BuilderMethods )
      {
         if( bm->PropEdit )
         {
            (*bm->PropEdit)( self, window, pens );
         }
         else
            printf( "no PropEdit for %s\n", ClassName( (struct PObject *)self ) );
      }
      else
         printf( "no builder method for %s\n", ClassName( (struct PObject *)self ) );
   }
}


void GiveItAName( PObject *self )
{
   static name_count = 0;
   short len, i;
   char number_string[10];
   char *name, *classname;

   /* clear count buf */
   memset( number_string, 0, 10 );

   sprintf( number_string, "%d", name_count++ );

   classname = ClassName( self );
   len = strlen( classname ) + strlen( number_string );

   name = (char*) Acalloc( len+1, 1 );
   if( name )
   {
      strcat( name, classname );
      strcat( name, number_string );

      for( i = 0; i < len; i++ )
         name[i] = tolower( name[i] );

      SetObjectName( self, name );

      Afree( name );
   }
}

BOOL InitImageBob( GraphicObject    *self,
                   ImageBob         *ibob )
   /* Create an image for drag&drop Bob. */
{
   struct GraphicObjectClass *class;
   struct BuilderMethods *bm;

   if( self )
   {
      if( class = (struct GraphicObjectClass *) self->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->InitImageBob )
               return (*bm->InitImageBob)( self, ibob );
         }
      }
   }
   return FALSE;
}

void CleanUpImageBob( GraphicObject *self,
                      ImageBob      *ibob )
   /* Deallocate any CHIP memory that was allocated by InitImageBob() */
{
   struct GraphicObjectClass *class;
   struct BuilderMethods *bm;

   if( self )
   {
      if( class = (struct GraphicObjectClass *) self->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->CleanUpImageBob )
            {
               (*bm->CleanUpImageBob)( self, ibob );
               return;
            }
         }
      }
   }
}

void WriteCode(  GraphicObject         *self,
                 struct BuilderWindow  *bw,
                 FILE                  *file,
                 UWORD                  section )
{
   struct GraphicObjectClass *class;
   struct BuilderMethods *bm;

   if( self )
   {
      if( class = (struct GraphicObjectClass *) self->isa )
      {
         if( bm = class->BuilderMethods )
         {
            if( bm->WriteCode )
               (*bm->WriteCode)( self, bw, file, section );
            else
               printf( "No WriteCode method for %s\n", class->ClassName );
         }
         else printf( "No builder methods for %s.\n", class->ClassName );
      }
   }
}



#define FP fprintf( file,

void WriteFlags( FILE *file, ULONG flags, struct FlagPair *pairs, int n_pairs )
{
   short n = 0;
   short i;

   for( i = 0; i < n_pairs; i++ )
   {
      if( flags & pairs[i].Flag )
      {
         if( n )
            FP " | ");

         FP "%s", pairs[i].FlagName );
         n++;
      }
   }

   if( n == 0 ) FP "0");
}
