#include "GraphicObjectIO.h"
#include <string.h>
#include <graphics/gfx.h>
#include <intuition/intuition.h>
#include "PObject.h"
#include "PObjectClass.h"
#include "AmigaMem.h"
#include "BuilderMethods.h"

struct GfxObj_filerep /* Representation of an outlinebox in a file. */
{
   short    NameLen;    /* Length of ObjName string. */
   short    TitleLen;   /* Length of Title string */
   Point    location;
   Point    size;
};



long WriteGfxObjChunk( GraphicObject *GfxObj, FILE *file, long Id )
{
   long beginkey, status;
   struct GfxObj_filerep pfr;
   char *name, *title;

   name = PObjectName( (struct PObject *)GfxObj );
   title = Title( GfxObj );
   pfr.location  = Location( GfxObj );
   pfr.size      = Size( GfxObj );

   pfr.NameLen    = name  ? strlen( name )  : 0;
   pfr.TitleLen   = title ? strlen( title ) : 0;

   /* pfr_dump( pfr ); */

   status = beginkey = IFF_BeginChunk( file, Id );

   if( status >= 0 )
   {
      status = fwrite( &pfr, sizeof(struct GfxObj_filerep), 1, file );

      if( status >= 0 )
      {
         if( name )
            status = fwrite( name, pfr.NameLen, 1, file );

         if( status >= 0 )
         {
            if( title )
               status = fwrite( title, pfr.TitleLen, 1, file );

            if( status >= 0 )
            {
               status = IFF_EndChunk( file, beginkey );
            }
         }
      }

   }

   if( status < 0 )
      printf( "Error! status=%d\n", status );

   return status;
}



GraphicObject* ReadGfxObjChunk( PClass            *class,
                                FILE             *file,
                                struct ChunkInfo *chunk )
{
   struct GfxObj_filerep pfr;
   GraphicObject *GfxObj = NULL;
   long status;
   char *name = NULL;
   char *title = NULL;

   status = fread( &pfr, sizeof( pfr ), 1, file );

   /* pfr_dump( pfr ); */

   if( status > 0 )
   {
      if( pfr.NameLen )
      {
         name = Amalloc( pfr.NameLen+1 );
         fread( name, pfr.NameLen, 1, file );
      }

      if( pfr.TitleLen )
      {
         title = Amalloc( pfr.TitleLen+1 );
         fread( title, pfr.TitleLen, 1, file );
      }

      if( GfxObj = NewFromClass( class ) )
      {
         SetObjectName( (struct PObject *)GfxObj, name );
         SetTitle( GfxObj, title );
         SetLocation( GfxObj, pfr.location.x, pfr.location.y );
         SetSize( GfxObj, pfr.size.x, pfr.size.y );
      }
      Afree( name );
      Afree( title );
   }

   return (GraphicObject *)GfxObj;
}
