#include "PanelIO.h"
#include <string.h>
#include <graphics/gfx.h>
#include <intuition/intuition.h>
#include "AmigaMem.h"
#include "BuilderMethods.h"
#include "OutlineBox.h"
#include "OutlineBoxIO.h"
#include "TitleBox.h"
#include "TitleBoxIO.h"
#include "BoolGadget.h"
#include "BoolGadgetio.h"
#include "CheckBox.h"
#include "CheckBoxIO.h"
#include "Positioner.h"
#include "PositionerIO.h"
#include "CycleGadget.h"
#include "CycleGadgetIO.h"
#include "VSlider.h"
#include "HSlider.h"
#include "VScroller.h"
#include "HScroller.h"
#include "StringGadget.h"
#include "StringGadgetIO.h"
#include "StringLister.h"
#include "StringListerIO.h"
#include "ListEditor.h"
#include "dumpwait.h"

#if 0  /* Very broken prototypes -- EDB */
struct GraphicObjectClass* OutlineBoxClass   ( void );
struct GraphicObjectClass* TitleBoxClass     ( void );
struct GraphicObjectClass* BoolGadgetClass   ( void );
struct GraphicObjectClass* CheckBoxClass     ( void );
struct GraphicObjectClass* HSliderClass      ( void );
struct GraphicObjectClass* VSliderClass      ( void );
struct GraphicObjectClass* HScrollerClass    ( void );
struct GraphicObjectClass* VScrollerClass    ( void );
struct GraphicObjectClass* CycleGadgetClass  ( void );
struct GraphicObjectClass* StringGadgetClass ( void );
struct GraphicObjectClass* IntegerGadgetClass( void );
struct GraphicObjectClass* ScrollingListClass( void );
struct GraphicObjectClass* ListEditorClass   ( void );
#endif

/* Correct prototypes -- EDB */
/* But why weren't they picked up without explicitly stating here? */

struct GraphicObjectClass* OutlineBoxClass   ( void );  /* OK */
struct GraphicObjectClass* TitleBoxClass     ( void );  /* OK */
struct ValuatorClass      *BoolGadgetClass   ( void );
struct ValuatorClass      *CheckBoxClass     ( void );
struct PositionerClass    *HSliderClass      ( void );
struct PositionerClass    *VSliderClass      ( void );
struct PositionerClass    *HScrollerClass    ( void );
struct PositionerClass    *VScrollerClass    ( void );
struct ValuatorClass      *CycleGadgetClass  ( void );
struct ValuatorClass      *StringGadgetClass ( void );
struct ValuatorClass      *IntegerGadgetClass( void );
struct StringListerClass *ScrollingListClass ( void );
struct StringListerClass *ListEditorClass    ( void );


struct Panel_filerep /* Representation of a panel in a file. */
{
   short    NameLen;    /* Length of ObjName string. */
   short    TitleLen;   /* Length of Title string */

   Point location;
   Point size, minsize, maxsize;

   ULONG    IDCMPflags;
   USHORT   WindowFlags;
   USHORT   PanelFlags;
};

void pfr_dump( struct Panel_filerep pfr )
{
   printf("NameLen=%d, TitleLen=%d, location=(%d,%d), size=(%d,%d)\n",
      pfr.NameLen, pfr.TitleLen, pfr.location.x, pfr.location.y,
      pfr.size.x, pfr.size.y );

   printf("  minsize=(%d,%d), maxsize=(%d,%d),\n",
      pfr.minsize.x, pfr.minsize.y, pfr.maxsize.x, pfr.maxsize.y );

   printf("  IDCMP=%d, Window=%d, Panel=%d\n", pfr.IDCMPflags,
      pfr.WindowFlags, pfr.PanelFlags );
}


int WritePanelChunk( FILE *file, BuilderWindow *panel )
{
   long beginkey, status;
   struct Panel_filerep pfr;
   char *name, *title;

   name = PObjectName( (struct PObject *)panel );
   title = Title( (struct GraphicObject *)panel );
   pfr.location = Location( (struct GraphicObject *)panel );
   pfr.size     = Size( (struct GraphicObject *)panel );
   pfr.minsize.x     = panel->pw.NewWindow.MinWidth;
   pfr.maxsize.x     = panel->pw.NewWindow.MaxWidth;
   pfr.minsize.y     = panel->pw.NewWindow.MinHeight;
   pfr.maxsize.y     = panel->pw.NewWindow.MaxHeight;
   pfr.IDCMPflags    = panel->IDCMPFlags;
   pfr.WindowFlags   = panel->pw.NewWindow.Flags;

   pfr.PanelFlags = 0;
   pfr.NameLen    = name  ? strlen( name )  : 0;
   pfr.TitleLen   = title ? strlen( title ) : 0;

   /* pfr_dump( pfr ); */

   status = beginkey = IFF_BeginChunk( file, PANL );

   if (status >= 0)
   {
      status = fwrite( &pfr, sizeof(struct Panel_filerep), 1, file );

      if (status >= 0)
      {
         status = fwrite( name, pfr.NameLen, 1, file );

         if (status >= 0)
         {
            status = fwrite( title, pfr.TitleLen, 1, file );

            if (status >= 0)
            {
               status = IFF_EndChunk( file, beginkey );
               if (status < 0)
                  printf("Error ending chunk\n");
            }
            else
               printf("Error writing title.\n");
         }
         else
            printf("Error writing name.\n");
      }
      else
         printf("Error writing panel.\n");

   }
   else
      printf("Error creating IFF chunk.\n");

   if (status < 0)
      printf("Error! status=%d\n", status );

   return status;
}


int WritePanelForm( FILE* file, BuilderWindow* panel )
{
   long beginkey, status;
   struct GraphicObject *obj;

   status = beginkey = IFF_BeginGroupChunk( file, FORM, PANL );
   if (status >= 0)
   {
      status = WritePanelChunk( file, panel );

      for (obj = panel->pw.FirstGraphic;
           obj != NULL;
           obj = obj->Next)
      {
         WriteIFF( obj, file );
      }

      for (obj = (GraphicObject*) panel->pw.FirstInteractor;
           obj != NULL;
           obj = obj->Next)
      {
         WriteIFF( obj, file );
      }

      if (status >= 0)
      {
         status = IFF_EndChunk( file, beginkey );
      }
   }
   return status;
}

int WritePrecogForm( FILE* file, BuilderWindow* panel )
{
   long beginkey, status;

   status = beginkey = IFF_BeginGroupChunk( file, FORM, PRCG );
   if (status >= 0)
   {
      status = WritePanelForm( file, panel );
      if (status >= 0)
      {
         status = IFF_EndChunk( file, beginkey );
      }
   }
   return status;
}


#define WINDOWGADGET_BITS (WINDOWCLOSE|WINDOWDRAG|WINDOWDEPTH|WINDOWSIZING)
#define WINDOWGADGET_MASK ~(WINDOWGADGET_BITS)


int ReadPanelChunk( FILE* file, BuilderWindow* panel )
{
   struct Panel_filerep pfr;
   long status;
   char *name = NULL;
   char *title = NULL;

   status = fread( &pfr, sizeof(pfr), 1, file );

   /* pfr_dump( pfr );  */

   if (status > 0)
   {
      if (pfr.NameLen)
      {
         name = Amalloc( pfr.NameLen+1 );
         fread( name, pfr.NameLen, 1, file );
      }

      if (pfr.TitleLen)
      {
         title = Amalloc( pfr.TitleLen+1 );
         fread( title, pfr.TitleLen, 1, file );
      }

      SetObjectName( (struct PObject *)panel, name );
      SetTitle( (struct GraphicObject *)panel, title );
      SetLocation( (struct GraphicObject *)panel, pfr.location.x, pfr.location.y );
      SetSize( (struct GraphicObject *)panel, pfr.size.x, pfr.size.y );
      panel->pw.NewWindow.MinWidth = pfr.minsize.x;
      panel->pw.NewWindow.MaxWidth = pfr.maxsize.x;
      panel->pw.NewWindow.MinHeight = pfr.minsize.y;
      panel->pw.NewWindow.MaxHeight = pfr.maxsize.y;
      panel->IDCMPFlags = pfr.IDCMPflags;
      panel->pw.NewWindow.Flags = pfr.WindowFlags;
      Afree( name );
      Afree( title );
   }

   return status;
}

int ReadPrecogForm( FILE *file, BuilderWindow *panel, struct ChunkInfo *info )
{
   struct ChunkInfo subchunk;
   long pos, status;

   for(;;)
   {
      status = pos = ftell(file);
      if (status < 0) break;

      if (pos > info->end) break;

      status = IFF_ReadChunkInfo( file, &subchunk );
      if (status < 0) break;

      if ((subchunk.Id == FORM) && (subchunk.GroupId == PANL))
      {
         status = ReadPanelForm( file, panel, &subchunk );
         if (status < 0) break;
      }
   }

   return status;
}

int ReadPrecogFile( FILE *file, BuilderWindow *panel )
{
   struct ChunkInfo subchunk;
   long status;

   status = IFF_ReadChunkInfo( file, &subchunk );
   if (status < 0) return status;

   if ((subchunk.Id == FORM) && (subchunk.GroupId == PRCG))
   {
      status = ReadPrecogForm( file, panel, &subchunk );
   }
   else
      status = -1;

   return status;
}

#define READIFF(class) ReadIFF((struct GraphicObjectClass*) class (), file, &subchunk )

int ReadPanelForm( FILE *file, BuilderWindow *panel, struct ChunkInfo *info )
{
   struct ChunkInfo subchunk;
   long pos, status;
   char chunkid[5];

   chunkid[4] = '\0';

   for(;;)
   {
      status = pos = ftell(file);
      if (status < 0) break;
      else status = 0;

      if (pos > info->end) break;

      status = IFF_ReadChunkInfo( file, &subchunk );
      if (status < 0) break;

#if 0
strncpy( chunkid, (char*) &subchunk.Id, 4 );
printf("reading %s chunk.\n", chunkid );
DUMPWAIT("\n");
#endif
      if (subchunk.Id == PANL)
      {
         status = ReadPanelChunk( file, panel );
         if (status < 0) break;
      }

      else if (subchunk.Id == OBOX)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(OutlineBoxClass) );
      }

      else if (subchunk.Id == TBOX)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(TitleBoxClass) );
      }

      else if (subchunk.Id == BOLG)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(BoolGadgetClass) );
      }

      else if (subchunk.Id == CHKG)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(CheckBoxClass) );
      }

      else if (subchunk.Id == VSLI)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(VSliderClass) );
      }

      else if (subchunk.Id == HSLI)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(HSliderClass) );
      }

      else if (subchunk.Id == VSCR)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(VScrollerClass) );
      }

      else if (subchunk.Id == HSCR)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(HScrollerClass) );
      }

      else if ((subchunk.Id == FORM) && (subchunk.GroupId == CYCG))
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(CycleGadgetClass) );
      }

      else if (subchunk.Id == STRG)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(StringGadgetClass) );
      }

      else if (subchunk.Id == INTG)
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(IntegerGadgetClass) );
      }

      else if ((subchunk.Id == FORM) && (subchunk.GroupId == SCRL))
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(ScrollingListClass) );
      }

      else if ((subchunk.Id == FORM) && (subchunk.GroupId == LSED))
      {
         AddWindowPObject( (struct pcgWindow *)panel, READIFF(ListEditorClass) );
      }

      /* seek to next chunk */
      fseek( file, subchunk.next, SEEK_SET );

   }

   return status;
}

