#include "PositionerIO.h"
#include <string.h>
#include <graphics/gfx.h>
#include <intuition/intuition.h>
#include "PObject.h"
#include "PObjectClass.h"
#include "AmigaMem.h"
#include "BuilderMethods.h"

struct Posr_filerep /* Representation of an outlinebox in a file. */
{
   short    NameLen;    /* Length of ObjName string. */
   short    TitleLen;   /* Length of Title string */
   Point    location;
   Point    size;
   long     value;
   long     knobsize;
};



long WritePosrChunk( GraphicObject *obj, FILE *file, long Id )
{
   long beginkey, status;
   struct Posr_filerep pfr;
   char *name, *title;
   Positioner* posr = (Positioner *)obj;

   name = PObjectName( (struct PObject *)posr );
   title = Title( (struct GraphicObject *)posr );
   pfr.location  = Location( (struct GraphicObject *)posr );
   pfr.size      = Size( (struct GraphicObject *)posr );
   pfr.NameLen    = name  ? strlen( name )  : 0;
   pfr.TitleLen   = title ? strlen( title ) : 0;
   pfr.value      = Value( posr );
   pfr.knobsize   = KnobSize( posr );

   /* pfr_dump( pfr ); */

   status = beginkey = IFF_BeginChunk( file, Id );

   if( status >= 0 )
   {
      status = fwrite( &pfr, sizeof( struct Posr_filerep ), 1, file );

      if( status >= 0 )
      {
         status = fwrite( name, pfr.NameLen, 1, file );

         if( status >= 0 )
         {
            status = fwrite( title, pfr.TitleLen, 1, file );

            if( status >= 0 )
            {
               status = IFF_EndChunk( file, beginkey );
               if( status < 0 )
                  printf( "Error ending chunk\n" );
            }
            else
               printf( "Error writing title.\n" );
         }
         else
            printf( "Error writing name.\n" );
      }
      else
         printf( "Error writing posr.\n" );

   }
   else
      printf( "Error creating IFF chunk.\n" );

   if( status < 0 )
      printf( "Error! status=%d\n", status );

   return status;
}



GraphicObject* ReadPosrChunk( PClass            *class,
                              FILE             *file,
                              struct ChunkInfo *chunk )
{
   struct Posr_filerep pfr;
   GraphicObject *Posr = NULL;
   long status;
   char *name = NULL;
   char *title = NULL;

   status = fread( &pfr, sizeof( pfr ), 1, file );

   /* pfr_dump( pfr );  */

   if( status > 0 )
   {
      if( pfr.NameLen )
      {
         name = Amalloc( pfr.NameLen +1 );
         fread( name, pfr.NameLen, 1, file );
      }

      if( pfr.TitleLen )
      {
         title = Amalloc( pfr.TitleLen +1 );
         fread( title, pfr.TitleLen, 1, file );
      }

      if( Posr = NewFromClass( class ) )
      {
         SetObjectName( (struct PObject *)Posr, name );
         SetTitle( (struct GraphicObject *)Posr, title );
         SetLocation( (struct GraphicObject *)Posr, pfr.location.x, pfr.location.y );
         SetSize( (struct GraphicObject *)Posr, pfr.size.x, pfr.size.y );
         SetKnobSize( (Positioner *)Posr, pfr.knobsize );
         SetValue( (Positioner *)Posr, pfr.value );
      }
      Afree( name );
      Afree( title );
   }

   return (GraphicObject *)Posr;
}
