#include "StringGadgetIO.h"
#include <string.h>
#include <graphics/gfx.h>
#include <intuition/intuition.h>
#include "AmigaMem.h"
#include "BuilderMethods.h"
#include <string.h>
#include "StringGadgetClass.h"
#include "IntegerGadgetClass.h"

struct StringGadget_filerep /* Representation of an outlinebox in a file. */
{
   short    NameLen;    /* Length of ObjName string. */
   short    TitleLen;   /* Length of Title string */
   Point    location;
   Point    size;
   long     nchars;
   long     value;
};

long WriteStringGadgetChunk( GraphicObject *gfxobj, FILE *file, long Id )
{
   long beginkey, status;
   struct StringGadget_filerep pfr;
   char *name, *title;
   BOOL is_intgad;
   char *valstring = NULL;
   StringGadget* self = (StringGadget *)gfxobj;
   struct StringInfo *stringinfo;

   stringinfo = (struct StringInfo *)self->g.SpecialInfo;
   is_intgad = isa( (struct PObject *)self, IntegerGadgetClass() );

   name           = PObjectName( (struct PObject *)self );
   title          = Title( (struct GraphicObject *)self );
   pfr.location  = Location( (struct GraphicObject *)self );
   pfr.size      = Size( (struct GraphicObject *)self );

   pfr.NameLen    = name  ? strlen( name )  : 0;
   pfr.TitleLen   = title ? strlen( title ) : 0;
   pfr.nchars     = stringinfo->MaxChars-1;

   if( is_intgad )
   {
      pfr.value = Value( (Valuator *)self );
   }
   else
   {
      valstring = StringValue( (Valuator *)self );

      pfr.value = valstring ? strlen( valstring ) : 0;
   }


   /* pfr_dump( pfr ); */

   status = beginkey = IFF_BeginChunk( file, Id );

   if( status >= 0 )
   {
      status = fwrite( &pfr, sizeof( struct StringGadget_filerep ), 1, file );

      if( status >= 0 )
      {
         if( name )
            status = fwrite( name, pfr.NameLen, 1, file );

         if( status >= 0 )
         {
            if( title )
               status = fwrite( title, pfr.TitleLen, 1, file );

            if( status >= 0 )
            {
               if( ! is_intgad )
               {
                  if( valstring )
                     fwrite( valstring, pfr.value, 1, file );
               }

               status = IFF_EndChunk( file, beginkey );
               if( status < 0 )
                  printf( "Error ending chunk\n" );
            }
            else
               printf( "Error writing title.\n" );
         }
         else
            printf( "Error writing name.\n" );
      }
      else
         printf( "Error writing self.\n" );

   }
   else
      printf( "Error creating IFF chunk.\n" );

   if( status < 0 )
      printf( "Error! status=%d\n", status );



   return status;
}



GraphicObject* ReadStringGadgetChunk( PClass            *class,
                                      FILE             *file,
                                      struct ChunkInfo *chunk )
{
   struct StringGadget_filerep pfr;
   long status;
   char *name = NULL;
   char *title = NULL;
   BOOL  is_intgad = ( class == IntegerGadgetClass() );
   char *valstring = NULL;
   StringGadget* self = NULL;
   struct StringInfo *stringinfo;

   status = fread( &pfr, sizeof( pfr ), 1, file );

   /* pfr_dump( pfr ); */

   if( status > 0 )
   {
      if( pfr.NameLen )
      {
         name = Amalloc( pfr.NameLen +1 );
         fread( name, pfr.NameLen, 1, file );
      }

      if( pfr.TitleLen )
      {
         title = Amalloc( pfr.TitleLen +1 );
         fread( title, pfr.TitleLen, 1, file );
      }

      if( ( ! is_intgad ) && ( pfr.value > 0 ) )
      {
         valstring = Amalloc( pfr.value+1 );
         fread( valstring, pfr.value, 1, file );
      }

      if( self = NewFromClass( class ) )
      {
         SetObjectName( (struct PObject *)self, name );
         SetTitle( (struct GraphicObject *)self, title );
         SetLocation( (struct GraphicObject *)self, pfr.location.x, pfr.location.y );
         SetSize( (struct GraphicObject *)self, pfr.size.x, pfr.size.y );

         stringinfo = (struct StringInfo *)self->g.SpecialInfo;

         Afree( stringinfo->Buffer );
         stringinfo->Buffer = Acalloc( pfr.nchars +1, 1 );
         stringinfo->MaxChars = pfr.nchars+1;

         if( is_intgad )
            SetValue( (Valuator *)self, pfr.value );
         else
            SetStringValue( (Valuator *)self, valstring );
      }
      Afree( name );
      Afree( title );
      Afree( valstring );
   }

   return (GraphicObject *)self;
}
