#include "ValuatorIO.h"
#include <string.h>
#include <graphics/gfx.h>
#include <intuition/intuition.h>
#include "PObject.h"
#include "PObjectClass.h"
#include "AmigaMem.h"
#include "BuilderMethods.h"

struct Valuator_filerep /* Representation of an outlinebox in a file. */
{
   short    NameLen;    /* Length of ObjName string. */
   short    TitleLen;   /* Length of Title string */
   Point    location;
   Point    size;
   long     value;
};



long WriteValuatorChunk( GraphicObject *gfxobj, FILE *file, long Id )
{
   long beginkey, status;
   struct Valuator_filerep pfr;
   char *name, *title;

   name = PObjectName( (struct PObject *)gfxobj );
   title = Title( gfxobj );
   pfr.location  = Location( gfxobj );
   pfr.size      = Size( gfxobj );

   pfr.NameLen    = name  ? strlen( name )  : 0;
   pfr.TitleLen   = title ? strlen( title ) : 0;
   pfr.value      = Value( (Valuator *)gfxobj );

   /* pfr_dump( pfr ); */

   status = beginkey = IFF_BeginChunk( file, Id );

   if( status >= 0 )
   {
      status = fwrite( &pfr, sizeof( struct Valuator_filerep ), 1, file );

      if( status >= 0 )
      {
         status = fwrite( name, pfr.NameLen, 1, file );

         if( status >= 0 )
         {
            status = fwrite( title, pfr.TitleLen, 1, file );

            if( status >= 0 )
            {
               status = IFF_EndChunk( file, beginkey );
               if( status < 0 )
                  printf( "Error ending chunk\n" );
            }
            else
               printf( "Error writing title.\n" );
         }
         else
            printf( "Error writing name.\n" );
      }
      else
         printf( "Error writing gfxobj.\n" );

   }
   else
      printf( "Error creating IFF chunk.\n" );

   if( status < 0 )
      printf( "Error! status=%d\n", status );

   return status;
}



GraphicObject* ReadValuatorChunk( PClass            *class,
                                  FILE             *file,
                                  struct ChunkInfo *chunk )
{
   struct Valuator_filerep pfr;
   GraphicObject *gfxobj = NULL;
   long status;
   char *name = NULL;
   char *title = NULL;

   status = fread( &pfr, sizeof( pfr ), 1, file );

   /* pfr_dump( pfr ); */

   if( status > 0 )
   {
      if( pfr.NameLen )
      {
         name = Amalloc( pfr.NameLen +1 );
         fread( name, pfr.NameLen, 1, file );
      }

      if( pfr.TitleLen )
      {
         title = Amalloc( pfr.TitleLen +1 );
         fread( title, pfr.TitleLen, 1, file );
      }

      if( gfxobj = NewFromClass( class ) )
      {
         SetObjectName( (struct PObject *)gfxobj, name );
         SetTitle( gfxobj, title );
         SetLocation( gfxobj, pfr.location.x, pfr.location.y );
         SetSize( gfxobj, pfr.size.x, pfr.size.y );
         SetValue( (Valuator *)gfxobj, pfr.value );
      }
      Afree( name );
      Afree( title );
   }

   return (GraphicObject *)gfxobj;
}
