#ifndef BUILDERMETHODS_H
#define BUILDERMETHODS_H

#include <stdio.h>
#include "parms.h"  /* try to insure prototypes are expanded */
#include "GraphicObject.h"
#include "Precognition.h"
#include "Precognition3d.h"
#include "ImageBob.h"
#include "IFFio.h"
#include "BuilderWindow.h"
#include "PObject.h" /* prototypes for SetObjectName() and isa() -- EDB */

/* The BuilderMethods structure contains pointers-to-functions which
 * are used by the interface builder program.
 */


typedef struct BuilderMethods
{
   void  *(*New)    ( PObject *self );

   void   (*Free)   ( PObject *self );

   void (*PropEdit)
      ( GraphicObject      *self,
        struct pcgWindow   *window,
        pcg_3DPens          pens );
                     /* self MUST be attached to an open window. */

   BOOL (*InitImageBob)
      ( GraphicObject         *self,
        ImageBob              *ibob );

   void (*CleanUpImageBob)
      (  GraphicObject        *self,
         ImageBob             *ibob );

   void (*WriteCode)
      (  GraphicObject        *self,
         struct BuilderWindow *bw,
         FILE                 *file,
         UWORD                 section );

   long (*WriteIFF)
      (  GraphicObject        *self,
         FILE                 *file );

   GraphicObject* (*ReadIFF)
      (  PClass             *class,
         FILE              *file,
         struct ChunkInfo  *chunk );

   /* More methods as I think of them. */
} BuilderMethods;


void *New    ( PObject *self );
   /* This duplicates an object, creating it from the heap.
    */

void *NewFromClass( struct GraphicObjectClass *class );
   /* Same as New(), but you supply a class instead of an object. */


void  Free   ( PObject *self );
   /* Deallocates an object.  object MUST have been dynamically
    * allocated, or Mr. Guru will visit you.
    */


void PropEdit( GraphicObject      *self,
               struct pcgWindow   *window,
               pcg_3DPens          pens );

   /* PropEdit() calls the object-specific GUI editor for
    * that object.
    */


void GiveItAName( PObject *self );
    /* generates a default name for an object. */


BOOL InitImageBob( GraphicObject    *self,
                   ImageBob         *image );
   /* Create an image for drag&drop Bob. */

void CleanUpImageBob( GraphicObject *self,
                      ImageBob      *image );
   /* Deallocate any CHIP memory that was allocated by InitImage() */

void WriteCode(  GraphicObject        *self,
                 struct BuilderWindow *bw,
                 FILE                 *file,
                 UWORD                 section );

struct FlagPair
{
   ULONG  Flag;
   char  *FlagName;
};

void WriteFlags( FILE *file, ULONG flags, struct FlagPair *pairs, int n_pairs );
   /*
   ** This utility, given an array of FlagPairs, writes the code equivalent
   ** of the flag pairs to 'file'.
   */


long WriteIFF
      (  GraphicObject        *self,
         FILE                 *file );

GraphicObject* ReadIFF
      (  PClass             *class,
         FILE              *file,
         struct ChunkInfo  *chunk );


#endif
