#include "BuilderWindow_builder.h"

#include <string.h>
#include "Checkbox.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "AmigaMem.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif

#include "AutoCleanUp.h"
#include "Project.h"

extern char silly;
#define WAITCHAR scanf( "%c\n", &silly )
#define DUMPWAIT(s) printf( s ); WAITCHAR


typedef struct BuilderWindow_PropEditWindow
{
   pcgWindow      window;

   CheckBox       WindowClose,
                  WindowDepth,
                  WindowDrag,
                  WindowSize;

   StringGadget   PObjectName, /* Window's Source code variable name. */
                  Title;       /* title of window. */

   OutlineBox     System_Gadget_Outline,
                  Window_Size_Outline;

   IntegerGadget  MinWidth,
                  MaxWidth,
                  Width,
                  MinHeight,
                  MaxHeight,
                  Height,
                  LeftEdge,
                  TopEdge;

   BoolGadget     bg_OK,
                  bg_Cancel;

} BuilderWindow_PropEditWindow;

void bwpew_CleanUp( BuilderWindow_PropEditWindow *self )
{
   CleanUp( (struct PObject *)&self->window );
   CleanUp( (struct PObject *)&self->WindowClose );
   CleanUp( (struct PObject *)&self->WindowDepth );
   CleanUp( (struct PObject *)&self->WindowDrag );
   CleanUp( (struct PObject *)&self->WindowSize );
   CleanUp( (struct PObject *)&self->PObjectName );
   CleanUp( (struct PObject *)&self->Title );
   CleanUp( (struct PObject *)&self->System_Gadget_Outline );
   CleanUp( (struct PObject *)&self->Window_Size_Outline );
   CleanUp( (struct PObject *)&self->MinWidth );
   CleanUp( (struct PObject *)&self->MaxWidth );
   CleanUp( (struct PObject *)&self->Width );
   CleanUp( (struct PObject *)&self->MinHeight );
   CleanUp( (struct PObject *)&self->MaxHeight );
   CleanUp( (struct PObject *)&self->Height );
   CleanUp( (struct PObject *)&self->LeftEdge );
   CleanUp( (struct PObject *)&self->TopEdge );
   CleanUp( (struct PObject *)&self->bg_OK );
   CleanUp( (struct PObject *)&self->bg_Cancel );
}



#define SELF &self->window

void GracefulExit( short ExitStatus );

void BuilderWindow_PropEditWindow_Init( BuilderWindow_PropEditWindow *self,
                                        Screen                       *screen,
                                        pcg_3DPens                    pens )
{
   pcgWindow_Init( SELF, 20,20, 410, 200, 1, 1, 65535, 65535,
      "Window Properties",
      CLOSEWINDOW | REFRESHWINDOW,
      WINDOWDEPTH | WINDOWDRAG | WINDOWCLOSE | ACTIVATE,
      screen );

   StringGadget_Init( &self->PObjectName, 150, 15, 230, 80, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   OutlineBox_Init( &self->System_Gadget_Outline, 10, 35, 390, 50, pens, " System Gadgets " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->System_Gadget_Outline );

   CheckBox_Init( &self->WindowClose, 20, 45, pens, "Close", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->WindowClose );

   CheckBox_Init( &self->WindowDepth, 120, 45, pens, "Depth", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->WindowDepth );

   CheckBox_Init( &self->WindowDrag, 220, 45, pens, "Drag", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->WindowDrag );

   CheckBox_Init( &self->WindowSize, 320, 45, pens, "Size", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->WindowSize );

   StringGadget_Init( &self->Title, 150, 65, 230, 80, pens, "Window Title:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Title );

   OutlineBox_Init( &self->Window_Size_Outline, 10, 92, 390, 85, pens, " Window Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Window_Size_Outline );

   IntegerGadget_Init( &self->LeftEdge, 150, 100, 80, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, 300, 100, 80, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   IntegerGadget_Init( &self->Width, 150, 120, 80, 6, pens, "Initial size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Width );

   IntegerGadget_Init( &self->Height, 300, 120, 80, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Height );

   IntegerGadget_Init( &self->MinWidth, 150, 140, 80, 6, pens, "Minimum size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->MinWidth );

   IntegerGadget_Init( &self->MinHeight, 300, 140, 80, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->MinHeight );

   IntegerGadget_Init( &self->MaxWidth, 150, 160, 80, 6, pens, "Maximum size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->MaxWidth );

   IntegerGadget_Init( &self->MaxHeight, 300, 160, 80, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->MaxHeight );

   BoolGadget_Init( &self->bg_OK, 10, 180, 80, 14, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel, 320, 180, 80, 14, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}




void BuilderWindow_SetSettings( pcgWindow *self, BuilderWindow_PropEditWindow *w )
{
   USHORT wflags;
   Point location, size;

   wflags = self->NewWindow.Flags;

   SetValue( (Valuator *)&w->WindowClose, ( wflags & WINDOWCLOSE ) );
   SetValue( (Valuator *)&w->WindowDepth, ( wflags & WINDOWDEPTH ) );
   SetValue( (Valuator *)&w->WindowDrag,  ( wflags & WINDOWDRAG ) );
   SetValue( (Valuator *)&w->WindowSize,  ( wflags & WINDOWSIZING ) );

   SetStringValue( (Valuator *)&w->Title, Title( (struct GraphicObject *)self ) );
   SetStringValue( (Valuator *)&w->PObjectName, PObjectName( (struct PObject *)self ) );

   size     = Size( (struct GraphicObject *)self );
   location = Location( (struct GraphicObject *)self );
   SetValue( (Valuator *)&w->LeftEdge,  location.x );
   SetValue( (Valuator *)&w->TopEdge,   location.y );
   SetValue( (Valuator *)&w->Width,     size.x );
   SetValue( (Valuator *)&w->Height,    size.y );
   SetValue( (Valuator *)&w->MinWidth,  self->NewWindow.MinWidth  );
   SetValue( (Valuator *)&w->MinHeight, self->NewWindow.MinHeight );
   SetValue( (Valuator *)&w->MaxWidth,  self->NewWindow.MaxWidth  );
   SetValue( (Valuator *)&w->MaxHeight, self->NewWindow.MaxHeight );

}

#define NO_SYSGADGETS ( ~( WINDOWCLOSE|WINDOWDEPTH|WINDOWDRAG|WINDOWSIZING ) )



void BuilderWindow_PutSettings( pcgWindow *self, BuilderWindow_PropEditWindow *w )
{
   USHORT wflags;
   ULONG  iflags;
   Window* is_open;
   char *title;


   /* DUMPWAIT("putsettings\n");  */

   is_open = iWindow( self );


   if( is_open )
   {
      pcgCloseWindow( self );
   }

   wflags = self->NewWindow.Flags & NO_SYSGADGETS;
   iflags = REFRESHWINDOW;


   if( Value( (Valuator *)&w->WindowClose ) )
   {
      wflags |= WINDOWCLOSE;
      iflags |= CLOSEWINDOW;
   }
   if( Value( (Valuator *)&w->WindowDepth ) )
   {
      wflags |= WINDOWDEPTH;
   }
   if( Value( (Valuator *)&w->WindowDrag  ) )
   {
      wflags |= WINDOWDRAG;
   }
   if( Value( (Valuator *)&w->WindowSize  ) )
   {
      wflags |= WINDOWSIZING;
      iflags |= NEWSIZE;
   }


   self->NewWindow.Flags = wflags;
   self->IDCMPFlags      = iflags;

   /* DUMPWAIT( "about to set title\n" ); */


   title = StringValue( (Valuator *)&w->Title );
   SetTitle( (struct GraphicObject *)self, title );

   /* DUMPWAIT( "about to set name\n" ); */

   /* PObject name, assumed to be dynamically allocated */
   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );

   /* DUMPWAIT( "about to set size and location\n" ); */

   SetLocation( (struct GraphicObject *)self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetSize    ( (struct GraphicObject *)self, Value( (Valuator *)&w->Width ), Value( (Valuator *)&w->Height ) );
   self->NewWindow.MinWidth  = Value( (Valuator *)&w->MinWidth );
   self->NewWindow.MinHeight = Value( (Valuator *)&w->MinHeight );
   self->NewWindow.MaxWidth  = Value( (Valuator *)&w->MaxWidth );
   self->NewWindow.MaxHeight = Value( (Valuator *)&w->MaxHeight );


   /* DUMPWAIT( "about to reopen\n" );  */

   /* if it was open, re-open it. */
   if( is_open )
   {
      if( ! pcgOpenWindow( self ) )
         GracefulExit( FALSE );
   }
   /* DUMPWAIT( "end Put settings\n" ); */

   Project_Modify();
}


BuilderWindow_PropEditWindow bw_PropWindow;
BOOL bw_PropWindow_Initialized = FALSE;

void BuilderWindow_PropEdit( BuilderWindow *self,
                             pcgWindow     *window,
                             pcg_3DPens     pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */
   Screen *screen;

   BOOL keep_going = TRUE;

   screen = self->pw.Window->WScreen;

   if( ! bw_PropWindow_Initialized )
   {
      BuilderWindow_PropEditWindow_Init( &bw_PropWindow, screen, pens );
      bw_PropWindow.window.SharedUserPort = self->pw.SharedUserPort;
      bw_PropWindow_Initialized = TRUE;

      RegisterCleanUp( (ptr2func)bwpew_CleanUp, &bw_PropWindow );
   }

   /* set settings. */
   BuilderWindow_SetSettings( &self->pw, &bw_PropWindow );

   if( ! pcgOpenWindow( &bw_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( &bw_PropWindow.window );

   Refresh( (struct Interactor *)&bw_PropWindow.window );
   Activate( (struct Interactor *)&bw_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *) GetMsg(iwindow->UserPort);
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *) GetMsg(iwindow->UserPort);
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if(event.Class == REFRESHWINDOW )
      {
         pcgWindow *some_window;

         some_window = (pcgWindow*) event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&bw_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            BuilderWindow_PutSettings( &self->pw, &bw_PropWindow );
            break;
         }

         if( Respond( (struct Interactor *)&bw_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&bw_PropWindow.Title, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.Title );

         if( Respond( (struct Interactor *)&bw_PropWindow.PObjectName, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&bw_PropWindow.LeftEdge, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&bw_PropWindow.TopEdge, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.TopEdge );

         if( Respond( (struct Interactor *)&bw_PropWindow.Width, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.Width );

         if( Respond( (struct Interactor *)&bw_PropWindow.Height, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.Height );

         if( Respond( (struct Interactor *)&bw_PropWindow.MinWidth, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.MinWidth );

         if( Respond( (struct Interactor *)&bw_PropWindow.MinHeight, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.MinHeight );

         if( Respond( (struct Interactor *)&bw_PropWindow.MaxWidth, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.MaxWidth );

         if( Respond( (struct Interactor *)&bw_PropWindow.MaxHeight, &event ) & DEACTIVATED)
            ActivateNext( (struct Interactor *)&bw_PropWindow.MaxHeight );


         Respond( (struct Interactor *)&bw_PropWindow.WindowClose, &event );
         Respond( (struct Interactor *)&bw_PropWindow.WindowDepth, &event );
         Respond( (struct Interactor *)&bw_PropWindow.WindowDrag, &event );
         Respond( (struct Interactor *)&bw_PropWindow.WindowSize, &event );

      }
   }

   pcgCloseWindow( &bw_PropWindow.window );

}

