#include <stdio.h>
#include "BuilderWindow.h"
#include "CodeWriter.h"
#include "BuilderMethods.h"
#include "PObjectClass.h"

#define FP fprintf( file,
 /* #define FP printf(   */


#define N_idcmpflags 23

struct FlagPair WindowIDCMPFlags[ N_idcmpflags ] = {
  {SIZEVERIFY, "SIZEVERIFY"},
  {NEWSIZE, "NEWSIZE"},
  {REFRESHWINDOW, "REFRESHWINDOW"},
  {MOUSEBUTTONS, "MOUSEBUTTONS"},
  {MOUSEMOVE, "MOUSEMOVE"},
  {GADGETDOWN, "GADGETDOWN"},
  {GADGETUP, "GADGETUP"},
  {REQSET, "REQSET"},
  {MENUPICK, "MENUPICK"},
  {CLOSEWINDOW, "CLOSEWINDOW"},
  {RAWKEY, "RAWKEY"},
  {REQVERIFY, "REQVERIFY"},
  {REQCLEAR, "REQCLEAR"},
  {MENUVERIFY, "MENUVERIFY"},
  {NEWPREFS, "NEWPREFS"},
  {DISKINSERTED, "DISKINSERTED"},
  {DISKREMOVED, "DISKREMOVED"},
  {WBENCHMESSAGE, "WBENCHMESSAGE"},
  {ACTIVEWINDOW, "ACTIVEWINDOW"},
  {INACTIVEWINDOW, "INACTIVEWINDOW"},
  {DELTAMOVE, "DELTAMOVE"},
  {VANILLAKEY, "VANILLAKEY"},
  {INTUITICKS, "INTUITICKS"}
};

#define N_wflags 16

struct FlagPair WindowFlags[ N_wflags ] = {
  {WINDOWSIZING, "WINDOWSIZING"},
  {WINDOWDRAG, "WINDOWDRAG"},
  {WINDOWDEPTH, "WINDOWDEPTH"},
  {WINDOWCLOSE, "WINDOWCLOSE"},
  {SIZEBRIGHT, "SIZEBRIGHT"},
  {SIZEBBOTTOM, "SIZEBBOTTOM"},
  {SMART_REFRESH, "SMART_REFRESH"},
  {SIMPLE_REFRESH, "SIMPLE_REFRESH"},
  {SUPER_BITMAP, "SUPER_BITMAP"},
  {BACKDROP, "BACKDROP"},
  {REPORTMOUSE, "REPORTMOUSE"},
  {GIMMEZEROZERO, "GIMMEZEROZERO"},
  {BORDERLESS, "BORDERLESS"},
  {ACTIVATE, "ACTIVATE"},
  {RMBTRAP, "RMBTRAP"},
  {NOCAREREFRESH, "NOCAREREFRESH"}
};


void BuilderWindow_WriteCode(  BuilderWindow *self,
                               BuilderWindow *bw,
                               FILE          *file,
                               UWORD          section )
{
   char *name, *title;
   Point size, location;

   name = PObjectName( (struct PObject *)self );

  /* printf( "Write Code for %s\n", ClassName( self ) );  */


   switch( section )
   {
      case CODE_BANNER:
         FP "/* =====================================================\n");
         FP "**   Created by the Precognition Interface Builder\n");
         FP "** =====================================================\n" );
         FP "**\n");
         FP "**  Link this file with 'Precognition.lib'\n");
         FP "**\n");
         FP "*/\n\n");
         break;

      case CODE_INCLUDES:
         FP "#include \"Precognition.h\"\n" );
         FP "#include \"Intuition_Utils.h\"\n\n");
         FP "#include <proto/exec.h>\n");
         FP "#include <proto/intuition.h>\n");
         FP "#include <proto/graphics.h>\n");
         FP "#include <stdlib.h>\n");
         FP "#include <stdio.h>\n");
         FP "\n");
         break;

      case CODE_MAIN_GLOBALS:
         FP "\n" );
         FP "/*\n" );
         FP "** The event loop is terminated when 'Done' is true.\n" );
         FP "*/\n" );
         FP "\nBOOL Done = FALSE;\n\n");
         break;


      case CODE_BEGIN_WSTRUCT:
         FP "\nstruct %s\n", name );
         FP "{\n" );
         FP "   pcgWindow\t\tw;\n" );
         break;

      case CODE_MIDDLE_WSTRUCT:
         break;

      case CODE_END_WSTRUCT:
         FP "};\n\n");
         break;


      case CODE_PROTOS:
         FP "\n" );
         FP "/*=============== Prototypes ====================*/\n" );
         FP "\n" );
         FP "#ifdef ANSI_HEADERS\n" );
         FP "\n" );
         FP "   void  %s_Init( \n", name );
         FP "              struct %s *window,\n", name );
         FP "              struct Screen *screen );\n" );
         FP "\n" );
         FP "   void  %s_CleanUp( struct %s *window );\n", name, name );
         FP "\n" );
         FP "   ULONG %s_Respond( \n", name );
         FP "              struct %s *window,\n", name );
         FP "              struct IntuiMessage *event );\n" );
         FP "\n" );
#ifndef PMWCW
         FP "   void  %s_EventLoop( struct Screen *screen );\n", name );
         FP "\n" );
         FP "   void  GracefulExit( int ExitStatus );\n" );
         FP "\n" );
#endif
         FP "#else\n" );
         FP "\n" );
         FP "   void  %s_Init(); \n", name );
         FP "   void  %s_CleanUp();\n", name, name );
         FP "   ULONG %s_Respond();\n", name );
#ifndef PMWCW
         FP "   void  %s_EventLoop();\n", name );
         FP "   void  GracefulExit();\n" );
#endif
         FP "\n" );
         FP "#endif\n" );
         FP "\n" );
         FP "\n" );
         FP "\n" );

         break;


      case CODE_CTOR_FUNC:
         FP "void %s_Init( window, screen )\n", name );
         FP "   struct %s *window;\n", name );
         FP "   struct Screen *screen;\n" );
         FP "{\n");
         break;

      case CODE_CTOR_DECLS:
         FP "\n");
         FP "   pcg_3DPens    pens;\n" );
         break;

      case CODE_CTOR_INITS:
      /*   FP "\n   pens = StandardPens();\n\n" );  */
      /*
       * EDB -- changed to StandardScreenPens( screen )
       *        for automatic conversion on one bitplane screens
       *        to standard 2D style look
       */
         FP "\n   pens = StandardScreenPens( screen );\n\n" );
         break;

      case CODE_CTOR_BODY:
         location = Location( (struct GraphicObject *)&self->pw );
         size     = Size( (struct GraphicObject *)&self->pw );
         title    = Title( (struct GraphicObject *)&self->pw );

         FP "   /* Initialize the window */\n");
         FP "   pcgWindow_Init( &window->w,\n" );
         FP "      %d, %d, %d, %d, /* LeftEdge, TopEdge, Width, Height */\n",
            location.x, location.y, size.x, size.y );
         FP "      %d, %d, %d, %d, /* min width & height, max width & height */\n",
               self->pw.NewWindow.MinWidth, self->pw.NewWindow.MinHeight,
               self->pw.NewWindow.MaxWidth, self->pw.NewWindow.MaxHeight );

         if( title )
            FP "      \"%s\"", title );
         else
            FP "      NULL" );
         FP ", /* title */\n" );

         FP "      ");
         WriteFlags( file, self->IDCMPFlags, WindowIDCMPFlags, N_idcmpflags );
         FP ", /* IDCMPFlags */\n" );

         FP "      ");
         WriteFlags( file, self->pw.NewWindow.Flags, WindowFlags, N_wflags );
         FP ", /* Flags */\n" );

         FP "      screen );\n\n" );
         break;


      case CODE_CTOR_FUNC_END:
         FP "}\n\n\n");
         break;

      case CODE_DTOR_FUNC:
         FP "void %s_CleanUp( window )\n", name );
         FP "   struct %s *window;\n", name );
         FP "{\n");

         break;

      case CODE_DTOR_BODY:
         FP "   CleanUp( (struct PObject *)&window->w );\n" );
         break;


      case CODE_DTOR_FUNC_END:
         FP "}\n\n\n");
         break;


      case CODE_RESPOND_FUNC:
         FP "ULONG %s_Respond( window, event )\n", name );
         FP "     struct %s *window;\n", name );
         FP "     struct IntuiMessage *event;\n" );
         FP "{\n");
         break;


      case CODE_RESPOND_DECLS:
         FP "\n");
         FP "   ULONG response, r;\n");
         FP "   struct Window *intuition_window;\n");
         break;

      case CODE_RESPOND_INITS:
         FP "\n");
         FP "   response = 0L;\n");
         break;

      case CODE_SWITCH:
         FP "\n");
         FP "   /* check to see if 'event' happenned in this window */\n");
         FP "   intuition_window = iWindow( &window->w );\n" );
         FP "   if( event->IDCMPWindow == intuition_window )\n");
         FP "   {\n");
         FP "      switch( event->Class )\n");
         FP "      {\n\n");
         break;

      case CODE_CLOSEWINDOW_EVENT:
         FP "         case CLOSEWINDOW:\n");
         FP "            pcgCloseWindow( &window->w );\n" );
         FP "         \n" );
         FP "            Done =  TRUE; /* Trigger exit of the event loop */\n");
         FP "            break;\n\n");
         break;

      case CODE_REFRESHWINDOW_EVENT:
         FP "         case REFRESHWINDOW:\n");
         FP "            BeginRefresh( intuition_window );\n" );
         FP "            Refresh( (struct Interactor *)&window->w );\n" );
         FP "            EndRefresh( intuition_window, TRUE );\n" );
         FP "            break;\n\n");
         break;

      case CODE_DEFAULT_EVENT:

         FP "         default:\n");
         break;

      case CODE_DEFAULT_EVENT_END:
         FP "            break;\n\n");
         break;

      case CODE_END_SWITCH:
         FP "      } /* end switch */\n\n");
         break;

      case CODE_END_IF:
         FP "   } /* end if */\n\n");
         break;

      case CODE_RESPOND_END:
         FP "   return response;\n");
         FP "}\n");
         break;

      case CODE_DRIVER:
         FP "\n\n");
         FP "void %s_EventLoop ( screen )\n", name );
         FP "   struct Screen *screen;\n", name );
         FP "{\n" );
         FP "   struct %s %s;\n", name, name );
         FP "   struct Window *iw;              /* Intuition window */\n" );
         FP "   struct IntuiMessage *imsg, event;\n" );
         FP "   struct MsgPort *userport;\n" );
         FP "   \n" );
         FP "   \n" );
         FP "   %s_Init( &%s, screen );\n", name, name );
         FP "   \n" );
         FP "   if( ( iw = pcgOpenWindow( &%s.w ) ) != NULL )\n", name );
         FP "   {\n" );
         FP "      userport = iw->UserPort;\n" );
         FP "      \n" );
         FP "      /*====== Event Loop ================*/\n" );
         FP "      while( ! Done )\n" );
         FP "      {\n" );
         FP "         imsg = WaitForMessage( userport );\n" );
         FP "         event = *imsg;\n" );
         FP "         ReplyMsg( (struct Message *) imsg );\n" );
         FP "         \n" );
         FP "         %s_Respond( &%s, &event );\n", name, name );
         FP "         \n" );
         FP "      }\n" );
         FP "\n" );
         FP "      pcgCloseWindow( &%s.w );\n", name );
         FP "   }\n" );
         FP "   else /* Error : window couldn't be opened. */\n" );
         FP "   {\n" );
         FP "      ; /*** YOUR ERROR HANDLING HERE ***/\n" );
         FP "   }\n" );
         FP "\n" );
         FP "\n" );
         FP "   %s_CleanUp( &%s );\n", name, name );
         FP "}\n" );

         break;

      case CODE_MAIN:

         FP "\n\n");
         FP "#include <stdio.h>\n" );
         FP "#include <intuition/screens.h>\n" );
         FP "\n" );
         FP "/* declare lib structures */\n" );
         FP "struct IntuitionBase *IntuitionBase = NULL;\n" );
         FP "struct GfxBase       *GfxBase       = NULL;\n" );
         FP "\n" );
         FP "#define HAS_CONSOLE ( argc > 0 )\n" );
         FP "/*\n" );
         FP "** If 'argc' is greater than 0, than the program was started from\n" );
         FP "** the command line, and therefore has a console that one can\n" );
         FP "** 'fprintf' to.\n" );
         FP "**\n" );
         FP "** If 'argc' == 0, this was started from Workbench.  It's not\n" );
         FP "** safe to assume there's a console attached to programs started\n" );
         FP "** from Workbench.\n" );
         FP "*/\n" );
         FP "\n" );
         FP "struct Screen *screen;\n" );
         FP "\n" );
         FP "void main ( argc, argv )\n" );
         FP "   int argc;\n" );
         FP "   char **argv;\n" );
         FP "{\n" );
         FP "  \n" );
         FP "\n" );
         FP "   /* open intuition library */\n" );
         FP "   IntuitionBase = (struct IntuitionBase *)\n" );
         FP "      OpenLibrary( \"intuition.library\", 0 );\n" );
         FP "   if( IntuitionBase == NULL )\n" );
         FP "   {\n" );
         FP "      if( HAS_CONSOLE ) \n" );
         FP "         fprintf( stderr, \"Error: couldn't open intuition.library\\n\" );\n" );
         FP "      GracefulExit( 10 );\n" );
         FP "   }\n" );
         FP "\n" );
         FP "   GfxBase = (struct GfxBase *)\n" );
         FP "      OpenLibrary( \"graphics.library\", 0 );\n" );
         FP "   if( GfxBase == NULL )\n" );
         FP "   {\n" );
         FP "      if( HAS_CONSOLE )\n" );
         FP "         fprintf( stderr, \"Error: couldn't open graphics.library\\n\" );\n" );
         FP "      GracefulExit( 20 );\n" );
         FP "   }\n" );
         FP "\n" );
         FP "   /* Open a screen, if desired. */\n");
         FP "\n");
         FP "\n" );
         FP "   /* Now do the window event loop. */\n");
         FP "   %s_EventLoop( screen );\n", name );
         FP "\n" );
         FP "   GracefulExit( 0 );\n" );
         FP "\n" );
         FP "}\n" );
         FP "\n" );
         FP "\n" );
         FP "void GracefulExit( int status )\n" );
         FP "{\n" );
         FP "   \n" );
         FP "   if( GfxBase )         \n" );
         FP "   {\n" );
         FP "      CloseLibrary( (struct Library*) GfxBase );\n" );
         FP "      GfxBase = NULL;\n" );
         FP "   }\n" );
         FP "\n" );
         FP "   if( IntuitionBase )   \n" );
         FP "   {\n" );
         FP "      CloseLibrary( (struct Library*) IntuitionBase );\n" );
         FP "      IntuitionBase = NULL;\n" );
         FP "   }\n" );
         FP "\n" );
         FP "   exit( status );\n" );
         FP "}\n" );
         FP "\n" );

         break;

   }
}
