#include "BuilderMethods.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "CheckBox_Builder.h"
#include "PObject.h"
#include "PObjectClass.h"
#include <string.h>
#include "AmigaMem.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#endif

#include "Autocleanup.h"
#include "project.h"

/* Changed from CheckBox to void -- EDB */
void *CheckBox_New( CheckBox *self )
{
   CheckBox *new_bgadget = NULL;

   if( new_bgadget = (CheckBox *)Amalloc( sizeof( CheckBox ) ) )
   {
      if( self )
         CheckBox_Init(  new_bgadget,   self->Location.x, self->Location.y,
                            self->Pens, Title( (struct GraphicObject *)self ), (BOOL)Value( (Valuator *)self ) );
      else
         CheckBox_Init( new_bgadget, 0, 0, StandardPens(), NULL, (BOOL)0 );

      GiveItAName( (struct PObject *)new_bgadget );
   }
   return new_bgadget;
}

typedef struct CheckBox_PropEditWindow
{
   pcgWindow      window;

   StringGadget   PObjectName, /* Window's Source code variable name. */
                  Title;      /* title of window. */

   OutlineBox     Dimension_Outline;

   IntegerGadget  LeftEdge, TopEdge;

   BoolGadget     bg_OK,
                  bg_Cancel;

   CheckBox       Checked; /* Initial State. */

} CheckBox_PropEditWindow;



#define SELF &self->window

void GracefulExit( short ExitStatus );

char cp_windowtitle[80];
#define X0 10
#define X1 X0+120
#define X2 X1+100
#define Y1 20
#define Y2 Y1+20
#define Y3 Y2+20
#define Y4 Y3+10
#define Y5 Y4+20

#define OK_X 80
#define OK_Y 14
#define INTGADG_WIDTH 60

void CheckBox_PropEditWindow_Init(
      CheckBox_PropEditWindow        *self,
      CheckBox                       *checkbox,
      pcgWindow                      *window,
      pcg_3DPens                      pens )
{
   Screen *screen;
   Point window_size;

   screen = window->Window->WScreen;

   memset( cp_windowtitle, 0 ,80 );
   strcat( cp_windowtitle, ClassName( (struct PObject *)checkbox ) );
   strcat( cp_windowtitle, " Properties" );

   pcgWindow_Init( SELF, 20,20, 320, 138, 1, 1, (USHORT)-1, (USHORT)-1, cp_windowtitle,
      CLOSEWINDOW | REFRESHWINDOW,
      ACTIVATE | SIMPLE_REFRESH | WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG,
      screen );

   window_size = Size( (struct GraphicObject *)SELF );

   StringGadget_Init( &self->PObjectName, X1, Y1, 160, 32, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   StringGadget_Init( &self->Title, X1, Y2, 160, 80, pens, "Label:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Title );

   OutlineBox_Init( &self->Dimension_Outline, X0, Y3,
      window_size.x-X0*2, 51, pens, " Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Dimension_Outline );

   IntegerGadget_Init( &self->LeftEdge, X1, Y4, INTGADG_WIDTH, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, X2, Y4, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   CheckBox_Init( &self->Checked, X1, 87, pens,
      "Checked", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Checked );

   BoolGadget_Init( &self->bg_OK, X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel,
      window_size.x-OK_X-X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}

void CheckBox_PropEditWindow_CleanUp( CheckBox_PropEditWindow *self )
{
   printf( "CheckBox_PropEditWindow_CleanUp\n" );

   CleanUp( (struct PObject *)&self->window );
   CleanUp( (struct PObject *)&self->PObjectName );
   CleanUp( (struct PObject *)&self->Title );
   CleanUp( (struct PObject *)&self->Dimension_Outline );
   CleanUp( (struct PObject *)&self->LeftEdge );
   CleanUp( (struct PObject *)&self->TopEdge );
   CleanUp( (struct PObject *)&self->Checked );
   CleanUp( (struct PObject *)&self->bg_OK );
   CleanUp( (struct PObject *)&self->bg_Cancel );
}



void CheckBox_SetSettings( CheckBox *self, CheckBox_PropEditWindow *w )
{
   Point location, size;
   USHORT sizeflags;

   SetStringValue( (Valuator *)&w->Title, Title( (struct GraphicObject *)self ) );
   SetStringValue( (Valuator *)&w->PObjectName, PObjectName( (struct PObject *)self ) );

   size      = Size( (struct GraphicObject *)self );
   location  = Location( (struct GraphicObject *)self );
   sizeflags = SizeFlags( (struct GraphicObject *)self );

   SetValue( (Valuator *)&w->LeftEdge,  location.x );
   SetValue( (Valuator *)&w->TopEdge,   location.y );

   SetValue( (Valuator *)&w->Checked, Value( (Valuator *)self ) );

}


void CheckBox_PutSettings(  CheckBox                *self,
                            CheckBox_PropEditWindow *w )
{
   char *title;

   /* title. (Assumed to be dynamically allocated */
   title = StringValue( (Valuator *)&w->Title );
   SetTitle( (struct GraphicObject *)self, (char *)title );

   /* PObject name, assumed to be dynamically allocated */
   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );
   SetLocation( (struct GraphicObject *)self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetValue( (Valuator *)self, Value( (Valuator *)&w->Checked ) );
   Project_Modify();

}


CheckBox_PropEditWindow CheckBox_PropWindow;
BOOL cb_PropWindow_Initialized = FALSE;


void CheckBox_PropEdit( CheckBox      *self,
                        pcgWindow     *window,
                        pcg_3DPens     pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */

   BOOL keep_going = TRUE;


   if( ! cb_PropWindow_Initialized )
   {
      cb_PropWindow_Initialized =  TRUE;
      CheckBox_PropEditWindow_Init( &CheckBox_PropWindow, self, window, pens );
      CheckBox_PropWindow.window.SharedUserPort = window->SharedUserPort;

      RegisterCleanUp( (ptr2func)CheckBox_PropEditWindow_CleanUp, &CheckBox_PropWindow );
   }

   /* set settings. */
   CheckBox_SetSettings( self, &CheckBox_PropWindow );

   if( ! pcgOpenWindow( &CheckBox_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( &CheckBox_PropWindow.window );

   Refresh( (struct Interactor *)&CheckBox_PropWindow.window );
   Activate( (struct Interactor *)&CheckBox_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if( event.Class == REFRESHWINDOW )
      {
         pcgWindow *some_window;

         some_window = (pcgWindow *)event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            CheckBox_PutSettings( self, &CheckBox_PropWindow );
            break;
         }

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.Title, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CheckBox_PropWindow.Title );

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.PObjectName, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CheckBox_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.LeftEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CheckBox_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&CheckBox_PropWindow.TopEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CheckBox_PropWindow.TopEdge );

         Respond( (struct Interactor *)&CheckBox_PropWindow.Checked, &event );

      }
   }

   pcgCloseWindow( &CheckBox_PropWindow.window );
}

