
#include "BuilderMethods.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "ListEditor.h"
#include "CycleGadget_Builder.h"
#include "minmax.h"
#include "pobject.h"
#include "PObjectClass.h"
#include "project.h"
#include <stdlib.h>
#include <string.h>
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#endif

#include "amigamem.h"
#include "autocleanup.h"
#include "dumpwait.h"

typedef struct CycleGadget_PropEditWindow
{
   pcgWindow      window;

   StringGadget   PObjectName, /* Window's Source code variable name. */
                  Title;      /* title of window. */

   OutlineBox     Dimensions,
                  Options;

   IntegerGadget  Width, Height, LeftEdge, TopEdge;

   CycleGadget    InitValue;

   BoolGadget     bg_OK,
                  bg_Cancel;

   ListEditor     Choices;

} CycleGadget_PropEditWindow;

void cgpew_CleanUp( CycleGadget_PropEditWindow *self )
{
   CleanUp( (struct PObject *)&self->PObjectName );
   CleanUp( (struct PObject *)&self->Title );
   CleanUp( (struct PObject *)&self->Dimensions );
   CleanUp( (struct PObject *)&self->Options );
   CleanUp( (struct PObject *)&self->Width );
   CleanUp( (struct PObject *)&self->Height );
   CleanUp( (struct PObject *)&self->LeftEdge );
   CleanUp( (struct PObject *)&self->TopEdge );
   CleanUp( (struct PObject *)&self->bg_OK );
   CleanUp( (struct PObject *)&self->bg_Cancel );
   CleanUp( (struct PObject *)&self->Choices );
   CleanUp( (struct PObject *)&self->InitValue );
   CleanUp( (struct PObject *)&self->window );
}


#define SELF &self->window

void GracefulExit( short ExitStatus );


#define INTGADG_WIDTH 60
#define X0 10
#define X1 (X0+130)
#define X2 (X1+90)
#define X3 (X2 + INTGADG_WIDTH + 20)
#define X4 (X3 + X0)
#define X5 (X4 + X0)

#define Y1 20
#define Y2 (Y1 + 20)
#define Y3 (Y2 + 25)
#define Y4 (Y3 + 10)
#define Y5 (Y4 + 20)
#define LIST_Y 100
#define Y1_5 (Y1 + 10)
#define Y6 (Y5 + 20)
#define OK_X 80
#define OK_Y 14

void CycleGadget_PropEditWindow_Init(
      CycleGadget_PropEditWindow        *self,
      CycleGadget                       *obj,
      pcgWindow                         *window,
      pcg_3DPens                         pens )
{
   Screen *screen;
   Point   window_size;

   screen = window->Window->WScreen;

   pcgWindow_Init( SELF, 20,20, 550, 157, 1, 1, (USHORT)-1, (USHORT)-1, "CycleGadget Properties",
      CLOSEWINDOW | REFRESHWINDOW,
      ACTIVATE | SIMPLE_REFRESH | WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG,
      screen );
   window_size = Size( (struct GraphicObject *)SELF );

   StringGadget_Init( &self->PObjectName, X1, Y1, 160, 32, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   StringGadget_Init( &self->Title, X1, Y2, 160, 80, pens, "Label:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Title );

   OutlineBox_Init( &self->Dimensions, X0, Y3,
     X3-X0, 70, pens, " Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Dimensions );

   IntegerGadget_Init( &self->LeftEdge, X1, Y4, INTGADG_WIDTH, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, X2, Y4, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   IntegerGadget_Init( &self->Width, X1, Y5, INTGADG_WIDTH, 6, pens, "Size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Width );

   IntegerGadget_Init( &self->Height, X2, Y5, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Height );

   CycleGadget_Init( &self->InitValue, X1, Y6, X2+INTGADG_WIDTH-X1, pens,
      "Initial value:", NULL );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->InitValue );

   OutlineBox_Init( &self->Options, X4, Y1,
     220, 115, pens, " Options " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Options );

   ListEditor_Init( &self->Choices, X5, Y1_5, 200, LIST_Y, pens, NULL );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Choices );

   BoolGadget_Init( &self->bg_OK, X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel,
      window_size.x-OK_X-X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}


void cg_HandleInitValue( CycleGadget                *self,
                         CycleGadget_PropEditWindow *w )
{
   StringList *cycle_stringlist, *edit_stringlist;
   long v;

   edit_stringlist  = StringList_of( (Valuator *)&w->Choices );
   cycle_stringlist = &w->InitValue.sl;

   StringList_DeleteAllStrings( cycle_stringlist );
   StringList_Dup ( edit_stringlist, cycle_stringlist );

   if( cycle_stringlist->nEntries > 0 )
   {
      v = Value( (Valuator *)self );
      v = MIN( v, cycle_stringlist->nEntries -1 );
      SetValue( (Valuator *)&w->InitValue, v );
      EnableIactor( (struct Interactor *)&w->InitValue, TRUE );
   }
   else
      EnableIactor( (struct Interactor *)&w->InitValue, FALSE );

}

void CycleGadget_SetSettings( CycleGadget                *self,
                              CycleGadget_PropEditWindow *w )
{
   Point       location, size;
   USHORT      sizeflags, i;
   StringList *sl;

   SetStringValue( (Valuator *)&w->Title, Title( (struct GraphicObject *)self ) );
   SetStringValue( (Valuator *)&w->PObjectName, PObjectName( (struct PObject *)self ) );

   size      = Size( (struct GraphicObject *)self );
   location  = Location( (struct GraphicObject *)self );
   sizeflags = SizeFlags( (struct GraphicObject *)self );

   SetValue( (Valuator *)&w->LeftEdge,  location.x );
   SetValue( (Valuator *)&w->TopEdge,   location.y );
   SetValue( (Valuator *)&w->Width,     size.x );
   SetValue( (Valuator *)&w->Height,    size.y );

   EnableIactor( (struct Interactor *)&w->Width,  (sizeflags & OBJSIZE_X));
   EnableIactor( (struct Interactor *)&w->Height, (sizeflags & OBJSIZE_Y));


   sl = &self->sl;
   for( i = 0; i < sl->nEntries; i++ )
   {
      AddString( (struct Interactor *)&w->Choices, sl->Entries[i], 0 );
   }

   cg_HandleInitValue( self, w );
}



void CycleGadget_PutSettings(  CycleGadget                *self,
                               CycleGadget_PropEditWindow *w )
{
   USHORT sizeflags;
   Point size;
   StringList *sl;

/* DUMPWAIT("CycleGadget_PutSettings\n"); */

   /* CycleGadget title. (Assumed to be dynamically allocated */
   SetTitle( (struct GraphicObject *)self, StringValue( (Valuator *)&w->Title ) );

/* DUMPWAIT("After setting title.\n");  */

   /* PObject name, assumed to be dynamically allocated */
   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );

/* DUMPWAIT("After setting name.\n"); */


   sizeflags = SizeFlags( (struct GraphicObject *)self );
   size = Size( (struct GraphicObject *)self );
   if( sizeflags & OBJSIZE_X )
      size.x = Value( (Valuator *)&w->Width );
   if( sizeflags & OBJSIZE_Y )
      size.y = Value( (Valuator *)&w->Height );

/* DUMPWAIT("After looking at sizeflags.\n"); */

   SetLocation( (struct GraphicObject *)self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetSize    ( (struct GraphicObject *)self, size.x, size.y );

/* DUMPWAIT("After setting Size&Location.\n"); */

   /* Now free the existing strings */
   sl = &self->sl;

   StringList_DeleteAllStrings( sl );
/* DUMPWAIT("After deleting strings.\n");  */
   StringList_Dup( StringList_of( (StringLister *)&w->Choices ), sl );
/* DUMPWAIT("After duping strings.\n"); */

   SetValue( (Valuator *)self, Value( (Valuator *)&w->InitValue ) );

/* DUMPWAIT("After SetValue.\n"); */
   Project_Modify();

}


CycleGadget_PropEditWindow CycleGadget_PropWindow;
BOOL cg_PropWindow_Initialized = FALSE;

void CycleGadget_PropEdit( CycleGadget *self,
                             pcgWindow     *window,
                             pcg_3DPens     pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */
   USHORT r;
   BOOL keep_going = TRUE;

   if( ! cg_PropWindow_Initialized )
   {
      cg_PropWindow_Initialized = TRUE;
      CycleGadget_PropEditWindow_Init( &CycleGadget_PropWindow, self,
         window, pens );
      CycleGadget_PropWindow.window.SharedUserPort = window->SharedUserPort;

      RegisterCleanUp( (ptr2func)cgpew_CleanUp, &CycleGadget_PropWindow );

   }

   /* set settings. */
   CycleGadget_SetSettings( self, &CycleGadget_PropWindow );

   if( ! pcgOpenWindow( &CycleGadget_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( &CycleGadget_PropWindow.window );

   Refresh( (struct Interactor *)&CycleGadget_PropWindow.window );
   Activate( (struct Interactor *)&CycleGadget_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *) GetMsg( iwindow->UserPort );
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if (event.Class == REFRESHWINDOW)
      {
         pcgWindow *some_window;

         some_window = (pcgWindow *)event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            CycleGadget_PutSettings( self, &CycleGadget_PropWindow );
            break;
         }


         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.Title, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.Title );

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.PObjectName, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.LeftEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.TopEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.TopEdge );

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.Width, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.Width );

         if( Respond( (struct Interactor *)&CycleGadget_PropWindow.Height, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.Height );

         if( ( r = Respond( (struct Interactor *)&CycleGadget_PropWindow.Choices, &event ) ) & CHANGED_STATE )
         {
            cg_HandleInitValue( self, &CycleGadget_PropWindow );
            Refresh( (struct Interactor *)&CycleGadget_PropWindow.InitValue );

            if( r & DEACTIVATED )
               ActivateNext( (struct Interactor *)&CycleGadget_PropWindow.Choices );
         }

         Respond( (struct Interactor *)&CycleGadget_PropWindow.InitValue, &event );
      }
   }

   pcgCloseWindow( &CycleGadget_PropWindow.window );

   DeleteAllStrings( (StringLister *)&CycleGadget_PropWindow.Choices );
}
