#include "BuilderMethods.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "GraphicObject_Builder.h"
#include "PObject.h"
#include "PObjectClass.h"
#include "AmigaMem.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/graphics_protos.h>
#include <clib/intuition_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#endif
#include <string.h>
#include "autocleanup.h"
#include "Project.h"

void go_InitBuilderMethods( BuilderMethods *bm )
{
   bm->New              = NULL;
   bm->Free             = (void(*)(PObject *))GraphicObject_Free;
   bm->PropEdit         = GraphicObject_PropEdit;
   bm->InitImageBob     = GraphicObject_InitImageBob;
   bm->CleanUpImageBob  = GraphicObject_CleanUpImageBob;
}

void GraphicObject_Free( GraphicObject *self )
{
   CleanUp( (struct PObject *)self );
   Afree( self );
}

#define WORDALIGN(n) ((n+15) & ~0x0F)
/* rounds 'n' up to the nearest 16. */


#define DEPTH 2

BOOL GraphicObject_InitImageBob( GraphicObject  *self,
                                 ImageBob       *ibob )
{
   Point     size, location;

   size     = Size( self );
   location = Location( self );

   /* printf("GraphicObject_InitImageBob for class \"%s\"\n", ClassName( self ) ); */

   /* Allocate memory for image. */
   if( ImageBob_Init( ibob, size.x, size.y, DEPTH ) )
   {
      /* printf( "Got memory OK\n" ); */
      SetLocation( self, 0, 0 );
      /* printf( "set location\n" ); */

      Render( self, ibob->ImagePort.rastport );
      /* printf( "rendered\n" ); */

      /* Draw shadow mask */
      SetDrMd ( ibob->ShadowPort.rastport, JAM1 );
      SetAPen ( ibob->ShadowPort.rastport, 1 );
      RectFill( ibob->ShadowPort.rastport, 0, 0, size.x -1, size.y -1 );
      /* printf( "shadow mask\n" ); */

      SetLocation( self, location.x, location.y ); /* restore location. */
      /* printf( "InitImageBob succeeded\n" ); */
      return TRUE;
   }
   printf( "InitImageBob failed\n" );
   return FALSE;
}


void GraphicObject_CleanUpImageBob( GraphicObject *self,
                                    ImageBob      *ibob )
{
   ImageBob_CleanUp( ibob );
}


typedef struct GraphicObject_PropEditWindow
{
   pcgWindow      window;

   StringGadget   PObjectName, /* Window's Source code variable name. */
                  Title;      /* title of window. */

   OutlineBox     Dimensions;

   IntegerGadget  Width, Height, LeftEdge, TopEdge;

   BoolGadget     bg_OK,
                  bg_Apply,
                  bg_Cancel;

} GraphicObject_PropEditWindow;


void gopew_CleanUp( GraphicObject_PropEditWindow *self )
{
   CleanUp( (struct PObject *)&self->window );
   CleanUp( (struct PObject *)&self->PObjectName );
   CleanUp( (struct PObject *)&self->Title );
   CleanUp( (struct PObject *)&self->Dimensions );
   CleanUp( (struct PObject *)&self->Width );
   CleanUp( (struct PObject *)&self->Height );
   CleanUp( (struct PObject *)&self->LeftEdge );
   CleanUp( (struct PObject *)&self->TopEdge );
   CleanUp( (struct PObject *)&self->bg_OK );
   CleanUp( (struct PObject *)&self->bg_Apply );
   CleanUp( (struct PObject *)&self->bg_Cancel );
}


#define SELF &self->window

void GracefulExit( short ExitStatus );

char go_windowtitle[80];

#define X0 10
#define X1 X0+120
#define X2 X1+100
#define Y1 20
#define Y2 Y1+20
#define Y3 Y2+20
#define Y4 Y3+10
#define Y5 Y4+20

#define OK_X 80
#define OK_Y 14
#define INTGADG_WIDTH 60

void GraphicObject_PropEditWindow_Init(
      GraphicObject_PropEditWindow        *self,
      GraphicObject                       *gfxobj,
      pcgWindow                           *window,
      pcg_3DPens                           pens )
{
   Screen *screen;
   Point   window_size;

   screen = window->Window->WScreen;

   pcgWindow_Init( SELF, 20,20, 320, 134, 1, 1, (USHORT)-1, (USHORT)-1, NULL,
      CLOSEWINDOW | REFRESHWINDOW,
      ACTIVATE | SIMPLE_REFRESH | WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG,
      screen );

   window_size = Size( (struct GraphicObject *)SELF );

   StringGadget_Init( &self->PObjectName, X1, Y1, 160, 32, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   StringGadget_Init( &self->Title, X1, Y2, 160, 80, pens, "Label:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Title );

   OutlineBox_Init( &self->Dimensions, X0, Y3,
     window_size.x-X0*2, 50, pens, " Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Dimensions );

   IntegerGadget_Init( &self->LeftEdge, X1, Y4, INTGADG_WIDTH, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, X2, Y4, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   IntegerGadget_Init( &self->Width, X1, Y5, INTGADG_WIDTH, 6, pens, "Size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Width );

   IntegerGadget_Init( &self->Height, X2, Y5, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Height );


   BoolGadget_Init( &self->bg_OK, X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel,
      window_size.x-OK_X-X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}



void GraphicObject_SetSettings( GraphicObject *self, GraphicObject_PropEditWindow *w )
{
   Point location, size;
   USHORT sizeflags;

   memset( go_windowtitle, 0 ,80 );
   strcat( go_windowtitle, ClassName( (struct PObject *)self ) );
   strcat( go_windowtitle, " Properties" );
   SetTitle( (struct GraphicObject *)&w->window, go_windowtitle );

   SetStringValue( (Valuator *)&w->Title, Title( (struct GraphicObject *)self ) );
   SetStringValue( (Valuator *)&w->PObjectName, PObjectName( (struct PObject *)self ) );

   size      = Size( (struct GraphicObject *)self );
   location  = Location( (struct GraphicObject *)self );
   sizeflags = SizeFlags( (struct GraphicObject *)self );

   SetValue( (Valuator *)&w->LeftEdge,  location.x );
   SetValue( (Valuator *)&w->TopEdge,   location.y );
   SetValue( (Valuator *)&w->Width,     size.x );
   SetValue( (Valuator *)&w->Height,    size.y );

   EnableIactor( (struct Interactor *)&w->Width,  ( sizeflags & OBJSIZE_X ) );
   EnableIactor( (struct Interactor *)&w->Height, ( sizeflags & OBJSIZE_Y ) );
}


void GraphicObject_PutSettings(  GraphicObject                *self,
                                 GraphicObject_PropEditWindow *w )
{
   USHORT sizeflags;
   Point size;

   /* Window title. (Assumed to be dynamically allocated */
   SetTitle( (struct GraphicObject *)self, StringValue( (Valuator *)&w->Title ) );
   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );


   sizeflags = SizeFlags( (struct GraphicObject *)self );
   size = Size( (struct GraphicObject *)self );
   if( sizeflags & OBJSIZE_X )
      size.x = Value( (Valuator *)&w->Width );
   if( sizeflags & OBJSIZE_Y )
      size.y = Value( (Valuator *)&w->Height );

   SetLocation( self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetSize    ( self, size.x, size.y );
   Project_Modify();

}

GraphicObject_PropEditWindow GraphicObject_PropWindow;
BOOL go_PropWindow_Initialized = FALSE;

void GraphicObject_PropEdit( GraphicObject *self,
                             pcgWindow     *window,
                             pcg_3DPens     pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */

   BOOL keep_going = TRUE;

   if( ! go_PropWindow_Initialized )
   {
      GraphicObject_PropEditWindow_Init( &GraphicObject_PropWindow, self, window, pens );
      GraphicObject_PropWindow.window.SharedUserPort = window->SharedUserPort;
      go_PropWindow_Initialized = TRUE;
      RegisterCleanUp( (ptr2func)gopew_CleanUp, &GraphicObject_PropWindow );
   }


   /* set settings. */
   GraphicObject_SetSettings( self, &GraphicObject_PropWindow );

   if (! pcgOpenWindow( &GraphicObject_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( (struct pcgWindow *)&GraphicObject_PropWindow.window );

   Refresh( (struct Interactor *)&GraphicObject_PropWindow.window );
   Activate( (struct Interactor *)&GraphicObject_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if( event.Class == REFRESHWINDOW )
      {
         pcgWindow *some_window;

         some_window = (pcgWindow *)event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            GraphicObject_PutSettings( self, &GraphicObject_PropWindow );
            break;
         }

#if 0
         if( Respond( &GraphicObject_PropWindow.bg_Apply, &event ) & CHANGED_STATE )
         {
            GraphicObject_PutSettings( self, &GraphicObject_PropWindow );
            Refresh( (struct Interactor *)window );  /* refresh this guy too. */
            break;
         }
#endif

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.Title, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.Title );

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.PObjectName, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.LeftEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.TopEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.TopEdge );

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.Width, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.Width );

         if( Respond( (struct Interactor *)&GraphicObject_PropWindow.Height, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&GraphicObject_PropWindow.Height );
      }
   }

   pcgCloseWindow( &GraphicObject_PropWindow.window );

}
