
#include "OutlineBox.h"
#include "GraphicObjectClass.h"
#include <stdlib.h>
#include <string.h>
#include "minmax.h"
#include <graphics/gfxmacros.h>
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include "amigamem.h"
#include "Precognition_Utils.h"
#include "BuilderMethods.h"
#include "GraphicObject_Builder.h"
#include "OutlineBox_coder.h"
#include "GraphicObjectIO.h"
#include "OutlineBoxIO.h"

/* Changed from OutlineBox to void -- EDB */
void *OutlineBox_New( OutlineBox *self )
{
   OutlineBox *new_box = NULL;

   if( new_box = Amalloc( sizeof( OutlineBox ) ) )
   {

      if( self )
         OutlineBox_Init( new_box, self->Location.x, self->Location.y,
            self->Size.x, self->Size.y, self->Pens, Title( (struct GraphicObject *)self ) );
      else
         OutlineBox_Init( new_box, 0, 0, 100, 50, StandardPens(), NULL );

      GiveItAName( (struct PObject *)new_box );

   }

   return new_box;
}

#define YSHIFT 5
#define FONT_WIDTH  8
#define FONT_HEIGHT 8
#define BASELINE    7

BOOL OutlineBox_InitImageBob( OutlineBox  *self,
                              ImageBob    *ibob )
{
   Point     size, location;
   UWORD     width, length, x, y;

   size     = Size( (struct GraphicObject *)self );
   y        = size.y + YSHIFT;
   location = Location( (struct GraphicObject *)self );

   /* Allocate memory for image. */
   if( ImageBob_Init( ibob, size.x, y, 2 ) )
   {

      SetLocation( (struct GraphicObject *)self, 0, 5 );

      Render( (struct GraphicObject *)self, ibob->ImagePort.rastport );

      /* Draw shadow mask */
      SetDrMd( ibob->ShadowPort.rastport, JAM2 );
      SetAPen( ibob->ShadowPort.rastport, 1 );
      SetBPen( ibob->ShadowPort.rastport, 1 );

      Move( ibob->ShadowPort.rastport, 0, YSHIFT );
      Draw( ibob->ShadowPort.rastport, size.x-1, YSHIFT );
      Draw( ibob->ShadowPort.rastport, size.x-1, y-1 );
      Draw( ibob->ShadowPort.rastport, 0, y-1 );
      Draw( ibob->ShadowPort.rastport, 0, YSHIFT-1 );

      Move( ibob->ShadowPort.rastport, 1, YSHIFT+1 );
      Draw( ibob->ShadowPort.rastport, size.x-2, YSHIFT+1 );
      Draw( ibob->ShadowPort.rastport, size.x-2, y-2 );
      Draw( ibob->ShadowPort.rastport, 1, y-2 );
      Draw( ibob->ShadowPort.rastport, 1, YSHIFT+1 );

      length   = strlen( self->Label );
      width    = length * FONT_WIDTH;
      x        = size.x/2 - width/2;
      y        = YSHIFT - FONT_HEIGHT/2 + BASELINE;

      Move( ibob->ShadowPort.rastport, x, y );
      Text( ibob->ShadowPort.rastport, self->Label, length );

      SetLocation( (struct GraphicObject *)self, location.x, location.y ); /* restore location. */
      return TRUE;
   }
   return FALSE;
}

struct BuilderMethods OutlineBox_bm;

void
patchOutlineBoxClass( void )
{
   struct GraphicObjectClass *class;
   struct GraphicObjectClass *OutlineBoxClass( void );


   class = OutlineBoxClass();

   go_InitBuilderMethods( &OutlineBox_bm );

   OutlineBox_bm.New          = (void *(*)(PObject *))OutlineBox_New;
   OutlineBox_bm.InitImageBob = (BOOL(*)(struct GraphicObject *,struct ImageBob *))OutlineBox_InitImageBob;
   OutlineBox_bm.WriteCode    = (void(*)(struct GraphicObject *,struct BuilderWindow *,FILE *,UWORD))OutlineBox_WriteCode;
   OutlineBox_bm.WriteIFF     = (long(*)(struct GraphicObject *,FILE *))WriteOBoxChunk;           /* problem compiling with DICE */
   OutlineBox_bm.ReadIFF      = ReadGfxObjChunk;

   class->BuilderMethods      = &OutlineBox_bm;

}
