#include "BuilderMethods.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "Positioner.h"
#include "Positioner_Builder.h"
#include "PObject.h"
#include "PObjectClass.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#include "amigamem.h"
#include "autocleanup.h"
#include "Project.h"
#include <string.h>

typedef struct Positioner_PropEditWindow
{
   pcgWindow      window;

   StringGadget   PObjectName; /* Window's Source code variable name. */
   StringGadget   Label;

   OutlineBox     Dimension_Outline;

   IntegerGadget  Width, Height, LeftEdge, TopEdge,
                  KnobPosition, KnobSize;

   BoolGadget     bg_OK,
                  bg_Cancel;

} Positioner_PropEditWindow;

void
ppew_CleanUp( Positioner_PropEditWindow *window )
{
   CleanUp( (struct PObject *)&window->window );
   CleanUp( (struct PObject *)&window->PObjectName );
   CleanUp( (struct PObject *)&window->Label );
   CleanUp( (struct PObject *)&window->Dimension_Outline );
   CleanUp( (struct PObject *)&window->Width );
   CleanUp( (struct PObject *)&window->Height );
   CleanUp( (struct PObject *)&window->LeftEdge );
   CleanUp( (struct PObject *)&window->TopEdge );
   CleanUp( (struct PObject *)&window->KnobPosition );
   CleanUp( (struct PObject *)&window->KnobSize );
   CleanUp( (struct PObject *)&window->bg_OK );
   CleanUp( (struct PObject *)&window->bg_Cancel );
}

#define SELF &self->window

void GracefulExit( short ExitStatus );

char po_windowtitle[80];

#define X0 10
#define X1 X0+130
#define X2 X1+160
#define Y1 20
#define Y2 Y1+20
#define Y3 Y2+20
#define Y4 Y3+10
#define Y5 Y4+20
#define Y6 Y5+20

#define OK_X 80
#define OK_Y 14
#define INTGADG_WIDTH 60

void Positioner_PropEditWindow_Init(
      Positioner_PropEditWindow *self,
      Positioner                *pos,
      pcgWindow                 *window,
      pcg_3DPens                 pens )
{
   Screen *screen;
   Point   window_size;

   screen = window->Window->WScreen;

   pcgWindow_Init( SELF, 20,20, 400, 158, 1, 1, (USHORT)-1, (USHORT)-1, NULL,
      CLOSEWINDOW | REFRESHWINDOW,
      ACTIVATE | SIMPLE_REFRESH | WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG,
      screen );
   window_size = Size( (struct GraphicObject *)SELF );

   StringGadget_Init( &self->PObjectName, X1, Y1, 160, 32, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   StringGadget_Init( &self->Label, X1, Y2, 160, 32, pens, "Title:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Label );

   OutlineBox_Init( &self->Dimension_Outline, X0, Y3,
     window_size.x-X0*2, 70, pens, " Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Dimension_Outline );

   IntegerGadget_Init( &self->LeftEdge, X1, Y4, INTGADG_WIDTH, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, X2, Y4, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   IntegerGadget_Init( &self->Width, X1, Y5, INTGADG_WIDTH, 6, pens, "Size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Width );

   IntegerGadget_Init( &self->Height, X2, Y5, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Height );

   IntegerGadget_Init( &self->KnobPosition, X1, Y6, INTGADG_WIDTH, 6, pens, "Knob Position:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->KnobPosition );

   IntegerGadget_Init( &self->KnobSize, X2, Y6, INTGADG_WIDTH, 6, pens, "Knob Size:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->KnobSize );


   BoolGadget_Init( &self->bg_OK, X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel,
      window_size.x-OK_X-X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}



void Positioner_SetSettings( Positioner *self, Positioner_PropEditWindow *w )
{
   Point location, size;
   USHORT sizeflags;
   USHORT knobsize, knobpos;

   memset( po_windowtitle, 0 ,80 );
   strcat( po_windowtitle, ClassName( (struct PObject *)self ) );
   strcat( po_windowtitle, " Properties" );
   SetTitle( (struct GraphicObject *)&w->window, po_windowtitle );


   SetStringValue( (Valuator *)&w->PObjectName, self->PObjectName );
   SetStringValue( (Valuator *)&w->Label, Title( (struct GraphicObject *)self ) );

   size        = Size( (struct GraphicObject *)self );
   location    = Location( (struct GraphicObject *)self );
   sizeflags   = SizeFlags( (struct GraphicObject *)self );
   knobsize    = KnobSize( self );
   knobpos     = Value( (Valuator *)self );

   SetValue( (Valuator *)&w->LeftEdge,       location.x );
   SetValue( (Valuator *)&w->TopEdge,        location.y );
   SetValue( (Valuator *)&w->Width,          size.x );
   SetValue( (Valuator *)&w->Height,         size.y );
   SetValue( (Valuator *)&w->KnobPosition,   knobpos );
   SetValue( (Valuator *)&w->KnobSize,       knobsize );

   EnableIactor( (struct Interactor *)&w->Width,  (sizeflags & OBJSIZE_X) );
   EnableIactor( (struct Interactor *)&w->Height, (sizeflags & OBJSIZE_Y) );
}


void Positioner_PutSettings(  Positioner                *self,
                              Positioner_PropEditWindow *w )
{
   USHORT sizeflags;
   Point  size;
   USHORT knobsize, knobpos;

   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );
   SetTitle( (struct GraphicObject *)self, StringValue( (Valuator *)&w->Label ) );


   sizeflags = SizeFlags( (struct GraphicObject *)self );
   size = Size( (struct GraphicObject *)self );
   if( sizeflags & OBJSIZE_X )
      size.x = Value( (Valuator *)&w->Width );
   if( sizeflags & OBJSIZE_Y )
      size.y = Value( (Valuator *)&w->Height );

   SetLocation( (struct GraphicObject *)self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetSize    ( (struct GraphicObject *)self, size.x, size.y );

   knobsize = Value( (Valuator *)&w->KnobSize );
   knobpos  = Value( (Valuator *)&w->KnobPosition );

   SetValue( (Valuator *)self, knobpos );
   SetKnobSize( self, knobsize );
   Project_Modify();
}


Positioner_PropEditWindow Positioner_PropWindow;
BOOL po_PropWindow_Initialized = FALSE;

void Positioner_PropEdit( Positioner   *self,
                          pcgWindow    *window,
                          pcg_3DPens    pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */
   Point size;

   BOOL keep_going = TRUE;


   if( ! po_PropWindow_Initialized )
   {
      po_PropWindow_Initialized = TRUE;
      Positioner_PropEditWindow_Init( &Positioner_PropWindow, self, window, pens );
      Positioner_PropWindow.window.SharedUserPort = window->SharedUserPort;
      RegisterCleanUp( (ptr2func)ppew_CleanUp, &Positioner_PropWindow );
   }

   /* set settings. */
   Positioner_SetSettings( self, &Positioner_PropWindow );

   if( ! pcgOpenWindow( &Positioner_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( &Positioner_PropWindow.window );

   Refresh( (struct Interactor *)&Positioner_PropWindow.window );
   Activate( (struct Interactor *)&Positioner_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if(event.Class == REFRESHWINDOW)
      {
         pcgWindow *some_window;

         some_window = (pcgWindow *)event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&Positioner_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            Positioner_PutSettings( self, &Positioner_PropWindow );
            break;
         }

         if( Respond( (struct Interactor *)&Positioner_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&Positioner_PropWindow.PObjectName, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&Positioner_PropWindow.Label, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.Label );

         if( Respond( (struct Interactor *)&Positioner_PropWindow.LeftEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&Positioner_PropWindow.TopEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.TopEdge );

         if( Respond( (struct Interactor *)&Positioner_PropWindow.Width, &event ) & DEACTIVATED )
         {
            size.x = Value( (Valuator *)&Positioner_PropWindow.Width );
            size.y = Value( (Valuator *)&Positioner_PropWindow.Height );
            size   = AskSize( (struct GraphicObject *)self, size.x, size.y );
            SetValue( (Valuator *)&Positioner_PropWindow.Width, size.x );
            SetValue( (Valuator *)&Positioner_PropWindow.Height, size.y );
            Refresh( (struct Interactor *)&Positioner_PropWindow.Width );
            Refresh( (struct Interactor *)&Positioner_PropWindow.Height );
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.Width );
         }

         if( Respond( (struct Interactor *)&Positioner_PropWindow.Height, &event ) & DEACTIVATED )
         {
            size.x = Value( (Valuator *)&Positioner_PropWindow.Width );
            size.y = Value( (Valuator *)&Positioner_PropWindow.Height );
            size   = AskSize( (struct GraphicObject *)self, size.x, size.y );
            SetValue( (Valuator *)&Positioner_PropWindow.Width, size.x );
            SetValue( (Valuator *)&Positioner_PropWindow.Height, size.y );
            Refresh( (struct Interactor *)&Positioner_PropWindow.Width );
            Refresh( (struct Interactor *)&Positioner_PropWindow.Height );
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.Height );
         }

         if( Respond( (struct Interactor *)&Positioner_PropWindow.KnobPosition, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.KnobPosition );

         if( Respond( (struct Interactor *)&Positioner_PropWindow.KnobSize, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&Positioner_PropWindow.KnobSize );
      }
   }

   pcgCloseWindow( &Positioner_PropWindow.window );

}
