#include "parms.h"
#include "BuilderMethods.h"
#include "StringGadget.h"
#include "OutlineBox.h"
#include "IntegerGadget.h"
#include "BoolGadget.h"
#include "ListEditor.h"
#include "CheckBox.h"
#include "ScrollingList.h"
#include "ScrollingList_Builder.h"
#include "PObject.h"
#include "PObjectClass.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#include "amigamem.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "AutoCleanUp.h"
#include "project.h"

typedef struct ScrollingList_PropEditWindow
{
   pcgWindow      window;

   StringGadget   PObjectName, /* Window's Source code variable name. */
                  Title;      /* title of window. */

   OutlineBox     Dimensions,
                  ListContents;

   IntegerGadget  Width, Height, LeftEdge, TopEdge;

   BoolGadget     bg_OK,
                  bg_Cancel;

   CheckBox       SelectMany;

   ListEditor     Choices;

} ScrollingList_PropEditWindow;


void
slpew_CleanUp( ScrollingList_PropEditWindow *window )
{
   CleanUp( (struct PObject *)&window->window      );
   CleanUp( (struct PObject *)&window->PObjectName );
   CleanUp( (struct PObject *)&window->Title       );
   CleanUp( (struct PObject *)&window->Dimensions  );
   CleanUp( (struct PObject *)&window->ListContents );
   CleanUp( (struct PObject *)&window->Width       );
   CleanUp( (struct PObject *)&window->Height      );
   CleanUp( (struct PObject *)&window->LeftEdge    );
   CleanUp( (struct PObject *)&window->TopEdge     );
   CleanUp( (struct PObject *)&window->bg_OK       );
   CleanUp( (struct PObject *)&window->bg_Cancel   );
   CleanUp( (struct PObject *)&window->SelectMany  );
   CleanUp( (struct PObject *)&window->Choices     );
}

#define SELF &self->window

void GracefulExit( short ExitStatus );

char sl_windowtitle[80];

#define INTGADG_WIDTH 60
#define X0 10
#define X1 X0+120
#define X2 X1+100
#define X3 X2 + INTGADG_WIDTH + 20
#define X4 X3 + X0
#define X5 X4 + X0

#define Y1 20
#define Y2 Y1 + 20
#define Y3 Y2 + 60
#define Y4 Y3 + 10
#define Y5 Y4 + 20
#define LIST_Y 100
#define Y1_5 Y1 + 10
#define Y6 Y1_5 + LIST_Y
#define OK_X 80
#define OK_Y 14

void ScrollingList_PropEditWindow_Init(
      ScrollingList_PropEditWindow        *self,
      ScrollingList                       *obj,
      pcgWindow                           *window,
      pcg_3DPens                           pens )
{
   Screen *screen;
   Point   window_size;

   screen = window->Window->WScreen;

   pcgWindow_Init( SELF, 20,20, 550, 172, 1, 1, (USHORT)-1, (USHORT)-1, NULL,
      CLOSEWINDOW | REFRESHWINDOW,
      ACTIVATE | SIMPLE_REFRESH | WINDOWCLOSE | WINDOWDEPTH | WINDOWDRAG,
      screen );
   window_size = Size( (struct GraphicObject *)SELF );

   StringGadget_Init( &self->PObjectName, X1, Y1, 160, 32, pens, "Variable Name:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->PObjectName );

   StringGadget_Init( &self->Title, X1, Y2, 160, 80, pens, "Label:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Title );

   OutlineBox_Init( &self->Dimensions, X0, Y3,
     X3-X0, 50, pens, " Dimensions " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Dimensions );

   IntegerGadget_Init( &self->LeftEdge, X1, Y4, INTGADG_WIDTH, 6, pens, "Location X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->LeftEdge );

   IntegerGadget_Init( &self->TopEdge, X2, Y4, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->TopEdge );

   IntegerGadget_Init( &self->Width, X1, Y5, INTGADG_WIDTH, 6, pens, "Size X:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Width );

   IntegerGadget_Init( &self->Height, X2, Y5, INTGADG_WIDTH, 6, pens, "Y:" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Height );

   OutlineBox_Init( &self->ListContents, X4, Y1,
     220, 130, pens, " List Contents " );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->ListContents );
                                                       /* Extra Argument??? -- EDB */
   ListEditor_Init( &self->Choices, X5, Y1_5, 200, LIST_Y, pens, NULL); /*, 80 ); */
   AddWindowPObject( SELF, (struct GraphicObject *)&self->Choices );

   CheckBox_Init( &self->SelectMany, X5, Y6, pens, "Select multiple?", FALSE );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->SelectMany );

   BoolGadget_Init( &self->bg_OK, X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, "  OK" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_OK );

   BoolGadget_Init( &self->bg_Cancel,
      window_size.x-OK_X-X0, window_size.y-OK_Y-4,
      OK_X, OK_Y, pens, " CANCEL" );
   AddWindowPObject( SELF, (struct GraphicObject *)&self->bg_Cancel );
}


void ScrollingList_SetSettings( ScrollingList *self, ScrollingList_PropEditWindow *w )
{
   Point       location, size;
   USHORT      sizeflags, i;
   StringList *sl;

   memset( sl_windowtitle, 0 ,80 );
   strcat( sl_windowtitle, ClassName( (struct PObject *)self ) );
   strcat( sl_windowtitle, " Properties" );
   SetTitle( (struct GraphicObject *)&w->window, sl_windowtitle );

   SetStringValue( (Valuator *)&w->Title, Title( (struct GraphicObject *)self ) );
   SetStringValue( (Valuator *)&w->PObjectName, self->slister.PObjectName );

   size      = Size( (struct GraphicObject *)self );
   location  = Location( (struct GraphicObject *)self );
   sizeflags = SizeFlags( (struct GraphicObject *)self );

   SetValue( (Valuator *)&w->LeftEdge,  location.x );
   SetValue( (Valuator *)&w->TopEdge,   location.y );
   SetValue( (Valuator *)&w->Width,     size.x );
   SetValue( (Valuator *)&w->Height,    size.y );

   if( isa( (struct PObject *)self, ScrollingListClass() ) )
   {
      SetValue( (Valuator *)&w->SelectMany, self->list.SelectMany );
      EnableIactor( (struct Interactor *)&w->SelectMany, TRUE );
      EnableIactor( (struct Interactor *)&w->Title, FALSE );
   }
   else
   {
      EnableIactor( (struct Interactor *)&w->SelectMany, FALSE );
      EnableIactor( (struct Interactor *)&w->Title, TRUE );
   }

   EnableIactor( (struct Interactor *)&w->Width,  ( sizeflags & OBJSIZE_X ) );
   EnableIactor( (struct Interactor *)&w->Height, ( sizeflags & OBJSIZE_Y ) );

   /* Now the tricky part: Copy the strings into the list-editor */
   sl = StringList_of( (StringLister *)self );
   DeleteAllStrings( (StringLister *)&w->Choices );
   for( i = 0; i < sl->nEntries; i++ )
   {
      AddString( (StringLister *)&w->Choices, sl->Entries[i], 0 );
   }
}


void ScrollingList_PutSettings(  ScrollingList                *self,
                                 ScrollingList_PropEditWindow *w )
{
   USHORT sizeflags;
   Point       size;
   USHORT      i;
   StringList *sl;


   SetTitle( (struct GraphicObject *)self, StringValue( (Valuator *)&w->Title ) );
   SetObjectName( (struct PObject *)self, StringValue( (Valuator *)&w->PObjectName ) );

   sizeflags = SizeFlags( (struct GraphicObject *)self );
   size = Size( (struct GraphicObject *)self );
   if( sizeflags & OBJSIZE_X )
      size.x = Value( (Valuator *)&w->Width );
   if( sizeflags & OBJSIZE_Y )
      size.y = Value( (Valuator *)&w->Height );

   SetLocation( (struct GraphicObject *)self, Value( (Valuator *)&w->LeftEdge ), Value( (Valuator *)&w->TopEdge ) );
   SetSize    ( (struct GraphicObject *)self, size.x, size.y );

   if( isa( (struct PObject *)self, ScrollingListClass() ) )
   {
      self->list.SelectMany = Value( (Valuator *)&w->SelectMany );
   }


   /* Now free the existing strings, and plug new strings in its place */
   DeleteAllStrings( (StringLister *)self );
   sl = StringList_of( (StringLister *)&w->Choices );
   for( i = 0; i < sl->nEntries; i++ )
   {
         AddString( (StringLister *)self, sl->Entries[i], 0 );
   }
   Project_Modify();
}


ScrollingList_PropEditWindow ScrollingList_PropWindow;
BOOL sl_PropWindow_Initialized = FALSE;

void ScrollingList_PropEdit( ScrollingList *self,
                             pcgWindow     *window,
                             pcg_3DPens     pens )
{
   IntuiMessage *Message, event;
   Window *iwindow; /* Intuition Window */

   BOOL keep_going = TRUE;


   if( ! sl_PropWindow_Initialized )
   {
      ScrollingList_PropEditWindow_Init( &ScrollingList_PropWindow, self, window, pens );
      ScrollingList_PropWindow.window.SharedUserPort = window->SharedUserPort;
      sl_PropWindow_Initialized = TRUE;

      RegisterCleanUp( (ptr2func)slpew_CleanUp, &ScrollingList_PropWindow );

   }
   /* set settings. */
   ScrollingList_SetSettings( self, &ScrollingList_PropWindow );

   if( ! pcgOpenWindow( &ScrollingList_PropWindow.window ) )
   {
      GracefulExit( FALSE );
   }
   iwindow = iWindow( (struct pcgWindow *)&ScrollingList_PropWindow.window );

   Refresh( (struct Interactor *)&ScrollingList_PropWindow.window );
   Activate( (struct Interactor *)&ScrollingList_PropWindow.PObjectName, TRUE );

   while( keep_going )
   {
      Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      if( Message == NULL )
      {
         WaitPort( iwindow->UserPort );
         Message = (struct IntuiMessage *)GetMsg( iwindow->UserPort );
      }
      event = *Message;
      ReplyMsg( (struct Message *)Message );


      if( event.Class == CLOSEWINDOW ) break;

      if( event.Class == REFRESHWINDOW )
      {
         pcgWindow *some_window;

         some_window = (pcgWindow *)event.IDCMPWindow->UserData;

         BeginRefresh( event.IDCMPWindow );
         Refresh( (struct Interactor *)some_window );
         EndRefresh( event.IDCMPWindow, TRUE );
      }

      else
      {

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.bg_OK, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            ScrollingList_PutSettings( self, &ScrollingList_PropWindow );
            break;
         }
#if 0
         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.bg_Apply, &event ) & CHANGED_STATE )
         {
            ScrollingList_PutSettings( self, &ScrollingList_PropWindow );
            Refresh( (struct Interactor *)window );  /* refresh this guy too. */
            break;
         }
#endif
         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.bg_Cancel, &event ) & CHANGED_STATE )
         {
            keep_going = FALSE;
            break;
         }

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.Title, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.Title );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.PObjectName, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.PObjectName );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.LeftEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.LeftEdge );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.TopEdge, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.TopEdge );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.Width, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.Width );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.Height, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.Height );

         if( Respond( (struct Interactor *)&ScrollingList_PropWindow.Choices, &event ) & DEACTIVATED )
            ActivateNext( (struct Interactor *)&ScrollingList_PropWindow.Choices );

         Respond( (struct Interactor *)&ScrollingList_PropWindow.SelectMany, &event );
      }
   }

   pcgCloseWindow( &ScrollingList_PropWindow.window );

   DeleteAllStrings( (StringLister *)&ScrollingList_PropWindow.Choices );
}
