#include "ClippedImageRastPort.h"
#ifndef __GNUC__
#include <clib/graphics_protos.h>
#include <clib/layers_protos.h>
#include <clib/exec_protos.h>
#endif
#ifdef __GNUC__
#include <proto/graphics.h>
#include <proto/layers.h>
#include <proto/exec.h>
#endif
#ifdef __SASC
#include <proto/graphics.h>
#include <proto/layers.h>
#include <proto/exec.h>
#endif

#define WORDALIGN(n) ( ( n + 15 ) & ~0x0F )
/* rounds 'n' up to the nearest 16. */

BOOL
ClippedImageRastPort_Init( struct ClippedImageRastPort *cirp,
                           struct Image                *image )
{
   UWORD planesize;  /* # of WORDS in one bitplane. */
   UWORD i;          /* loop index.                 */

   cirp->rastport  = NULL;
   cirp->layer     = NULL;
   cirp->layerinfo = NULL;

   planesize = ( WORDALIGN( image->Width ) / 16 ) * image->Height;

   InitBitMap(       &cirp->bitmap,
               (LONG)image->Depth,
               (LONG)image->Width,
               (LONG)image->Height );


   for( i = 0; i < 8; i++ ) /* make bitmap planes point to image data. */
   {
      if( i < image->Depth )
         cirp->bitmap.Planes[i] = (PLANEPTR)( &image->ImageData[ i * planesize ] );
      else
         cirp->bitmap.Planes[i] = NULL;
   }


   if( cirp->layerinfo = NewLayerInfo() )
   {
      if( cirp->layer = CreateUpfrontLayer( cirp->layerinfo,
                          &cirp->bitmap,
                           0,0,                             /* min X,Y      */
                           image->Width-1, image->Height-1, /* max X,Y      */
                           LAYERSIMPLE,                     /* Flags        */
                           NULL ) )                         /* Superbitmap. */
      {
         cirp->rastport = cirp->layer->rp;
         return TRUE;
      }
   }
/*
** FAILURE: if code reaches here, something failed.
*/
   ClippedImageRastPort_CleanUp( cirp );
   return FALSE;
}


void ClippedImageRastPort_CleanUp( struct ClippedImageRastPort *cirp )
{
   if( cirp->layer )
   {                /* Added cast to long -- EDB */
      DeleteLayer( (long)cirp->layerinfo, cirp->layer );
   }

   if( cirp->layerinfo )
   {
      DisposeLayerInfo( cirp->layerinfo );
   }

   cirp->rastport  = NULL;
   cirp->layer     = NULL;
   cirp->layerinfo = NULL;
}

