#include <stdlib.h>
#include <string.h>
#include "pcg_Menus.h"
#include "minmax.h"
#include "intuition_utils.h"
#ifndef __GNUC__
#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#endif
#ifdef __GNUC__
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif
#ifdef __SASC
#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#endif

void pcg_InitIntuiText( struct IntuiText *it,
                        char             *text,
                        USHORT            nchars,
                        struct TextAttr  *FontInfo )
{
   if( ! is_Workbench_v3() )
     {
      it->FrontPen  = 0;
      it->BackPen   = 1;
      it->DrawMode  = JAM2;
     }
   else /* 3.0 newlook menus */
     {
      it->FrontPen  = 1;
      it->BackPen   = 0;
      it->DrawMode  = JAM1;
     }

   it->TopEdge   = 1;
   it->ITextFont = FontInfo;
   it->IText     = text;
}


void pcg_InitTextMenuItem( pcg_TextMenuItem *item,
                           char             *text,
                           USHORT            nchars,
                           struct TextAttr  *FontInfo,
                           USHORT            FontWidth,
                           USHORT            FontHeight )
{
   item->LeftEdge    = 1;
   item->TopEdge     = 0;
   item->Width       = (nchars * FontWidth) + 10; /* 10 for padding */
   item->Height      = FontHeight;
   item->Flags       = ITEMENABLED | HIGHCOMP | ITEMTEXT;
   item->ItemFill    = (APTR) &item->iText;
   item->SelectFill  = NULL;

   pcg_InitIntuiText( &item->iText, text, nchars, FontInfo );
}

#define SEPARATOR -1

void
pcg_InitSeprMenuItem( pcg_SeprMenuItem *item )
{
   item->LeftEdge    = 1;
   item->TopEdge     = 0;
   item->Width       = 1;
   item->Height      = 4;
   item->Flags       = HIGHNONE;
   item->ItemFill    = (APTR) &item->bar;
   item->SelectFill  = NULL;
   item->ItemID      = SEPARATOR;

   item->bar.LeftEdge   = 0;
   item->bar.TopEdge    = 1;
   item->bar.Width      = 1;
   item->bar.Height     = 2;
   item->bar.Depth      = 1;
   item->bar.ImageData  = NULL;
   item->bar.PlanePick  = 0;
   item->bar.PlaneOnOff = 0;
   item->bar.NextImage  = NULL;
}

pcg_TextMenuItem *pcg_NewTextMenuItem( char            *text,
                                       USHORT           nchars,
                                       struct TextAttr *FontInfo,
                                       USHORT           FontWidth,
                                       USHORT           FontHeight )
{
   pcg_TextMenuItem *item  = NULL;

   if (item = (pcg_TextMenuItem*) calloc( sizeof(pcg_TextMenuItem), 1 ))
   {
      pcg_InitTextMenuItem( item, text, nchars, FontInfo, FontWidth, FontHeight );
   }
   return item;
}

pcg_SeprMenuItem *pcg_NewSeprMenuItem( void )
{
   pcg_SeprMenuItem *item  = NULL;

   if (item = (pcg_SeprMenuItem*) calloc( sizeof(pcg_SeprMenuItem), 1 ))
   {
      pcg_InitSeprMenuItem( item );
   }
   return item;
}



void pcg_FreeTextualMenuItems( pcg_MenuItem *item )
{
   pcg_MenuItem *next_item;

   while(item)
   {
      next_item = item->NextItem;

      pcg_FreeTextualMenuItems( item->SubItem );
      free( item );

      item = next_item;
   }
}

void pcg_AppendMenuItem( struct Menu *menu, pcg_MenuItem *item )
{
   pcg_MenuItem *i;

   if( menu->FirstItem )
   {  for ( i            = (pcg_MenuItem *) menu->FirstItem;
            i->NextItem != NULL;
            i            = i->NextItem )
        ;

      i->NextItem = item;
   }                      /* added cast -- EDB */
   else menu->FirstItem = (struct MenuItem *)item;
}

void pcg_AppendMenuSubItem( pcg_MenuItem *item, pcg_MenuItem *subitem )
{
   pcg_MenuItem *i;

   if (item->SubItem)
   {  for ( i            = item->SubItem;
            i->NextItem != NULL;
            i            = i->NextItem )
        ;

      i->NextItem = subitem;
   }
   else item->SubItem = subitem;
}


USHORT pcg_HeightOfItems( pcg_MenuItem *item )
/* NOTE: This assumes vertical stacking,
 * *i.e. just returns the sum-total of all item Heights.)
 */
{
   USHORT TotalHeight = 0;

   for ( ; item!=NULL; item = item->NextItem )
      TotalHeight += item->Height;

   return TotalHeight;
}


void pcg_StackItems( pcg_MenuItem  *item,
                     short          TopEdge,
                     BOOL           DoSubItemsToo )
/* Vertically stacks all menu items. */
{
   pcg_MenuItem *subitem;    /* SubItem                   */
   SHORT         subTopEdge; /* New top edge of subitems  */
   USHORT        subHeight;  /* Total height of subitems. */

   for ( ; item != NULL; item = item->NextItem )
   {
      item->TopEdge = TopEdge;

      /* If this item has any submenus, then fix their TopEdge. */
      if ((DoSubItemsToo) && (subitem = item->SubItem))
      {
         subHeight  = pcg_HeightOfItems( subitem );
         subTopEdge = MAX( (item->Height/2 -subHeight/2), -TopEdge );

         pcg_StackItems( subitem, subTopEdge, FALSE );
      }

      TopEdge      += item->Height;
   }
}


short pcg_MaxItemWidth( pcg_MenuItem *Item )
{
   short MaxWidth;
   MaxWidth = 0;
   while( Item!=NULL )
   {
      MaxWidth = MAX( Item->Width, MaxWidth );
      Item = Item->NextItem;
   }
   return MaxWidth;
}


#define SUBITEM_OVERLAP 48
   /* How much the subitems should overlap the parent. */

/* This gives all items in a chain a constant width  */
void pcg_TidyUpItemWidths( pcg_MenuItem *Item )
{
   struct pcg_MenuItem *i;
   SHORT MaxWidth;

   /* first find max width */
   MaxWidth = pcg_MaxItemWidth( Item );

   /* now Tidy up */
   for ( i=Item; i!=NULL; i=i->NextItem )
   {
      i->Width = MaxWidth;

      if (i->ItemID == SEPARATOR)
      {

         ((pcg_SeprMenuItem*)i)->bar.Width = MaxWidth;
      }

      /* If this item has any submenus, then move their LeftEdge. */
      if (i->SubItem)
      {
         pcg_MenuItem *subitem;       /* SubItem                   */
         SHORT         LeftEdge; /* New left edge of subitems */

         pcg_TidyUpItemWidths( i->SubItem );

         LeftEdge =  MaxWidth - SUBITEM_OVERLAP;
         for ( subitem  = i->SubItem;
               subitem != NULL;
               subitem  = subitem->NextItem )
            subitem->LeftEdge = LeftEdge;
      }
   }
}


#define SAFETY_FACTOR 5
   /* For some reason I haven't figured out, theres about a
   ** 10 pixel difference between the MenuItem width I calculate,
   ** and how much is really needed.
   */

void pcg_TidyUpItemLeftEdges( pcg_MenuItem  *FirstItem,
                              SHORT          desired_LeftEdge,
                              SHORT          D_to_RightEdge )

      /* 'D_to_RightEdge' = # of pixels to the right edge of screen. */
{
   SHORT width,  actual_LeftEdge, desired_SubItem_LeftEdge;
   pcg_MenuItem *i;

   if (!FirstItem) return;

   width = pcg_MaxItemWidth(FirstItem) + SAFETY_FACTOR;

   actual_LeftEdge = ((desired_LeftEdge+width) > D_to_RightEdge)
         ? D_to_RightEdge-width : desired_LeftEdge;

   for( i=FirstItem; i!=NULL; i=i->NextItem )
   {
      i->LeftEdge = actual_LeftEdge;
      if (i->SubItem)
      {
         desired_SubItem_LeftEdge = actual_LeftEdge + width
            - SUBITEM_OVERLAP;
         pcg_TidyUpItemLeftEdges( i->SubItem,
            desired_SubItem_LeftEdge, D_to_RightEdge );
      }
   }
}


void pcg_AppendMenu( struct Menu **FirstMenu, struct Menu *NewMenu )
{
   struct Menu *menu;
   SHORT  RightEdge;

/*   printf("pcg_AppendMenu( FirstMenu=%d, NewMenu=%d )\n",
      *FirstMenu, NewMenu );
*/
   /* find the last menu in the chain. */
   if (*FirstMenu)
   {
      for ( menu = *FirstMenu;
            menu->NextMenu != NULL;
            menu = menu->NextMenu );

      /* Initialize left edge of new menu */
      RightEdge         = menu->LeftEdge + menu->Width - 1;
      menu->NextMenu    = NewMenu;
      NewMenu->LeftEdge = RightEdge + 20;
   }
   else
   {
      *FirstMenu        = NewMenu;
      NewMenu->LeftEdge = 0;
   }

/*   printf("end pcg_AppendMenu( FirstMenu=%d, NewMenu=%d )\n",
      *FirstMenu, NewMenu );
*/

}

UWORD pcg_MenuNumber_of( struct Menu *menustrip, void *menu_or_item )
{

   struct Menu     *menu;
   struct MenuItem *item;
   struct MenuItem *subitem;
   UWORD     menu_count;
   UWORD     item_count;
   UWORD     subitem_count;

/*   printf("pcg_MenuNumber_of( menustrip=%d, menu_or_item=%d )\n",
      menustrip, menu_or_item );
*/
   for (menu  = menustrip, menu_count=0;
        menu != NULL;
        menu  = menu->NextMenu, menu_count++)
   {
      if (menu == menu_or_item)
      {
         return (UWORD) (SHIFTMENU(menu_count));
      }
      else
      {
         for ( item  = menu->FirstItem, item_count=0;
               item != NULL;
               item=item->NextItem, item_count++ )
         {
            if (item == menu_or_item)
            {
               return (UWORD) (SHIFTMENU(menu_count) |
                                 SHIFTITEM(item_count));
            }
            else
            {
               subitem_count = 0;
               for ( subitem  = item->SubItem, subitem_count=0;
                     subitem != NULL;
                     subitem  = subitem->NextItem, subitem_count++ )
               {
                  if (subitem == menu_or_item)
                     return (UWORD) (SHIFTMENU(menu_count)
                        | SHIFTITEM(item_count)
                        | SHIFTSUB(subitem_count));
               }
            }
         }
      }
   }

   return MENUNULL;
}
