#ifndef PCG_MENUS_H
#define PCG_MENUS_H

/* File "pcg_Menus.h" : Extensions to Intuition Menu items.
**
** Compiler: SAS C v5.10
** OS      : AmigaDOS 1.3/KickStart 1.2
**
** (c) Lee R Willis, August 1990, Mountain View, CA
*/

#ifndef STDLIB_H
#define STDLIB_H
#include <stdlib.h>
#endif

#ifndef STRING_H
#include <string.h>
#define STRING_H
#endif

#include <exec/types.h>
#include <intuition/intuition.h>

typedef struct pcg_MenuItem
{
   /*---- This part is compatible with a standard MenuItem ----*/
   struct pcg_MenuItem *NextItem;
   SHORT                LeftEdge,   TopEdge;
   SHORT                Width,      Height;
   USHORT               Flags;
   LONG                 MutualExclude;
   APTR                 ItemFill;
   APTR                 SelectFill;
   BYTE                 Command;
   struct pcg_MenuItem *SubItem;
   USHORT               NextSelect;

   /*------- New Stuff Here -------------------*/
   long                 ItemID;    /* Unique ID # */
   void                *UserData;  /* Free field for applications to chain
                                    * in special information.
                                    */
} pcg_MenuItem;
/* This is a handy-dandy extension to the MenuItem structure which
 * is Intuition compatible, and yet provides some extra fields which
 * I find handy.
 *
 * This way, you can have a unique ID which doesn't depend on the
 * position of the menu item.
 */

typedef struct pcg_TextMenuItem
{
   /*---- This part is compatible with a standard MenuItem ----*/
   struct pcg_MenuItem *NextItem;
   SHORT                LeftEdge,   TopEdge;
   SHORT                Width,      Height;
   USHORT               Flags;
   LONG                 MutualExclude;
   APTR                 ItemFill;
   APTR                 SelectFill;
   BYTE                 Command;
   struct pcg_MenuItem *SubItem;
   USHORT               NextSelect;

   /*------- New Stuff Here -------------------*/
   long                 ItemID;    /* Unique ID # */
   void                *UserData;  /* Free field for applications to chain
                                    * in special information.
                                    */

   struct IntuiText     iText;
} pcg_TextMenuItem;


typedef struct pcg_SeprMenuItem /* Separator bar. */
{
   /*---- This part is compatible with a standard MenuItem ----*/
   struct pcg_MenuItem *NextItem;
   SHORT                LeftEdge,   TopEdge;
   SHORT                Width,      Height;
   USHORT               Flags;
   LONG                 MutualExclude;
   APTR                 ItemFill;
   APTR                 SelectFill;
   BYTE                 Command;
   struct pcg_MenuItem *SubItem;
   USHORT               NextSelect;

   /*------- New Stuff Here -------------------*/
   long                 ItemID;    /* Unique ID # */
   void                *UserData;  /* Free field for applications to chain
                                    * in special information.
                                    */

   struct Image         bar;
} pcg_SeprMenuItem;



#define NEW_PCG_MENUITEM (pcg_MenuItem *) calloc(sizeof(pcg_MenuItem),1)


#define HEIGHT_OF_FONT 9 /* Width & Height of standard */
#define WIDTH_OF_FONT  8  /* Topaz 80 font.             */

void pcg_InitTextMenuItem( pcg_TextMenuItem  *item,
                           char              *text,
                           USHORT             nchars,
                           struct TextAttr   *FontInfo,
                           USHORT             FontWidth,
                           USHORT             FontHeight );

void pcg_InitSeprMenuItem( pcg_SeprMenuItem  *item );


pcg_TextMenuItem *pcg_NewTextMenuItem( char            *text,
                                       USHORT           nchars,
                                       struct TextAttr *FontInfo,
                                       USHORT           FontWidth,
                                       USHORT           FontHeight );
/* Returns a 1-line textual menu item, with the following flags:
 *    ITEMENABLED | HIGHCOMP | ITEMTEXT.
 *
 * Storage is taken from the Heap using 'malloc()', *not* 'AllocMem()'.
 *
 * The application needs to set the LeftEdge and TopEdge.
 * (See pcg_StackItems()).
 */

pcg_SeprMenuItem *pcg_NewSeprMenuItem( void );

#define PCG_NEWTEXTUALITEM(t) \
  pcg_NewTextualMenuItem(t, strlen(t), pcg_Topaz80, WIDTH_OF_FONT, HEIGHT_OF_FONT)
/* default way of building an item. */


void pcg_FreeTextualMenuItems( pcg_MenuItem *item );
/* Free()'s an entire chain of menu item that were allocated using
 * 'pcg_NewTextualMenuItem()'.
 */

void pcg_AppendMenu( struct Menu **FirstMenu, struct Menu *NewMenu );
/* Appends a Menu to a chain of menu items.  Note the double
 * stars (**) of FirstMenu.  To use this on a NewWindow structure,
 * pass '&(NewWindow->FirstMenu)'
 */

void pcg_AppendMenuItem( struct Menu *menu, pcg_MenuItem *item );
/* Adds a menu item to the tail of the list of menu items in 'menu'.
 * Does not do anything with the graphic location of the menu item.
 */

void pcg_AppendMenuSubItem( pcg_MenuItem *item, pcg_MenuItem *subitem );
/* Adds a menu subitem to the tail of the list of subitems.
 * Does not do anything with the graphic location of the menu item.
 */

void pcg_StackItems( pcg_MenuItem  *item,
                     short          TopEdge,
                     BOOL           DoSubItemsToo );
/* Vertically stacks a chain of menu items. */

void pcg_TidyUpItemWidths( pcg_MenuItem *Item );
/* Gives all menu items in a chain the same Width
 * (uses the maximum width of items.)
 */

void pcg_TidyUpItemLeftEdges( pcg_MenuItem  *FirstItem,
                              SHORT          desired_LeftEdge,
                              SHORT          D_to_RightEdge );
/* Sets the LeftEdges of a chain of menu items to prevent them
 * from exceeding the right-hand side of the screen.
 *
 * 'D_to_RightEdge' is the # of pixels from the LeftEdge of this
 * menu (or set of subitems) to the right side of the curent screen.
 */

UWORD pcg_MenuNumber_of( struct Menu *menustrip, void *menu_or_item );
/*
 * Given the address of a menustrip and the address of a particular menu
 * or item, this utility returns the MenuNumber of that menu or item.
 * (It does a tree search of the menu hierarchy looking for a match.)
 *
 *    I HATE MENU NUMBERS!
 *
 */

#endif
