#include "Editor.h"
#include "pcgWindow.h"
#include "pcgWindowClass.h"
#include "Menus/pcg_Menus.h"
#include "Menus/menustuff.h"
#include "EditBox_pointers.h"
#include "Intuition_Utils.h"
#include "MinMax.h"
#include "ChainedWindows.h"
#include "AmigaMem.h"
#include <graphics/gfxmacros.h>
#ifndef __GNUC__
#include <clib/intuition_protos.h>
#include <clib/exec_protos.h>
#include <clib/graphics_protos.h>
#endif
#ifdef __GNUC__
#include <proto/intuition.h>
#include <proto/exec.h>
#include <proto/graphics.h>
#endif
#ifdef __SASC
#include <proto/intuition.h>
#include <proto/exec.h>
#include <proto/graphics.h>
#endif
#include "BuilderWindow.h"
#include "BuilderMethods/buildermethods.h"
#include "project.h"
#include "funclists.h"

extern struct MinList DeferredResponses;

void Editor_BeSelfish( Editor *self );
void BuilderWindow_AddPObjectButDontRender( pcgWindow*, GraphicObject* );


#define KEYUP        0x80L
#define EITHER_SHIFT ( IEQUALIFIER_LSHIFT | IEQUALIFIER_RSHIFT )

#define EDGE_WIDTH   4 /* If User hits object within 4 pixels of edge, its
                        * considered a SizeHit. (i.e. start sizing.)
                        */

#define EraseLasso(s) Editor_DrawLasso( s )
#define DrawLasso(s) Editor_DrawLasso( s )

#define LASSO_PATS 8

extern char silly;
#define WAITCHAR scanf( "%c\n", &silly )
#define DUMPWAIT(s) printf( s ); WAITCHAR

/*
 * UWORD LassoPatterns[LASSO_PATS] = { 0x0F0F, 0x1E1E, 0x3C3C, 0x7878,
 *                                   0xF0F0, 0xE1E1, 0xC3C3, 0x8787 };
 */

UWORD LassoPatterns[LASSO_PATS] = { 0x8787, 0xc3c3, 0xe1e1, 0xf0f0,
                                    0x7878, 0x3c3c, 0x1e1e, 0x0f0f };

void ReportRightMouseButton( Window *w, BOOL Report )
{
   ULONG Flags;

   Flags = w->Flags;

   Forbid();
   if( Report )
   {
      w->Flags |= (ULONG)RMBTRAP;
   }
   else
   {
      w->Flags &= ~ ( (ULONG)RMBTRAP );
   }
   Permit();
}

/* For some stupid reason, ReportMouse is flaky. */
void ReportMouseMove( Window *w, BOOL Report )
{
   Forbid();
   if( Report )
      w->Flags |= (ULONG)REPORTMOUSE;
   else
      w->Flags &= ~( (ULONG)REPORTMOUSE );
   Permit();
}


void Editor_PropEdit( Editor *self, GraphicObject *gfxobj )
{
   pcgWindow   *pw;

   pw = InteractorWindow( (struct Interactor *)self );

   EnableAllChainedWindows( FALSE );      /* shut off all the usual windows */

/* DUMPWAIT("PropEdit\n"); */
   PropEdit( gfxobj, pw, self->pens );   /* edit the object. */
/* DUMPWAIT("end PropEdit\n"); */

   if( !isa( (struct PObject *)gfxobj, pcgWindowClass() ) ) /* do this for all objects except */
   {                                     /* the window itself.             */
      BeginRefresh( pw->Window );
      EndRefresh( pw->Window, TRUE );    /* Delete refresh clipping */
      Render( (struct GraphicObject *)pw, NULL );                /* Redraw the window       */
   }
   else
   {
      Refresh( (struct Interactor *)pw );                   /* draw changes            */
   }
   /* DUMPWAIT("enabling chained windows\n");  */
   EnableAllChainedWindows( TRUE );
   /* DUMPWAIT("end PropEdit\n"); */
}

HitClass ClassifyHit( GraphicObject *self, Point coord )
{
   USHORT hitclass = NONE;
   PIXELS xmin, xmax, ymin, ymax;
   tPoint location, size;
   USHORT sizeflags;


   /* printf("Classifying hit on object \"%s\"\n", self->PObjectName ); */

   location  = Location( self );
   size      = Size( self );
   sizeflags = SizeFlags( self );

   xmin = location.x;
   xmax = xmin + size.x -1;
   ymin = location.y;
   ymax = ymin + size.y -1;

   if( ( coord.x >= xmin ) && ( coord.x <= xmax ) &&
       ( coord.y >= ymin ) && ( coord.y <= ymax ) )
   {
      if( sizeflags & OBJSIZE_X )
      {
         if( coord.x < xmin+EDGE_WIDTH )
            hitclass |= LEFTHIT;
         else if( coord.x > xmax-EDGE_WIDTH )
            hitclass |= RIGHTHIT;
      }

      if( sizeflags & OBJSIZE_Y )
      {
         if( coord.y < ymin+EDGE_WIDTH )
            hitclass |= TOPHIT;

         else if( coord.y > ymax-EDGE_WIDTH )
            hitclass |= BOTTOMHIT;
      }

      if( hitclass == NONE ) /* didnt hit any edge. */
         hitclass = CENTERHIT;
   }

   else
      hitclass = NONE;


   return hitclass;
}

HitInfo FindHitPObject( GraphicObject *obj, Point coord )
{
   HitInfo hitinfo;

   hitinfo.hitclass = NONE;
   hitinfo.PObject   = NULL;


   for( ; obj != NULL; obj = obj->Next )
   {

      /* printf("Checking for hit on object \"%s\"\n", obj->PObjectName ); */

      hitinfo.hitclass = ClassifyHit( obj, coord );
      if( hitinfo.hitclass != NONE )
      {
         hitinfo.PObject = obj;
         break;
      }
   }

#if 0
   printf("FindHitPObject: hitclass=%d, PObject=", hitinfo.hitclass );
   if (hitinfo.PObject)
      printf("\"%s\"\n", hitinfo.PObject->PObjectName );
   else
      printf("\n");
#endif
   return hitinfo;
}

HitInfo FindHitSelection( SelectedObject *si, Point coord )
{
   HitInfo hitinfo;

   hitinfo.hitclass = NONE;
   hitinfo.PObject   = NULL;


   for( ; si != NULL; si = si->Next )
   {
      if( hitinfo.hitclass = ClassifyHit( si->PObject, coord ) )
      {
         hitinfo.PObject = si->PObject;
         break;
      }
   }

#if 0
   printf("FindHitSelection: hitclass=%d, PObject=", hitinfo.hitclass );
   if (hitinfo.PObject)
      printf("\"%s\"\n", hitinfo.PObject->PObjectName );
   else
      printf("\n");
#endif

   return hitinfo;
}

void Editor_DrawHighlights( Editor *self )
{
   RastPort *rport;

   rport = RPort( InteractorWindow( (struct Interactor *)self ) ) ;
   so_DrawHighlights( rport, self->FirstSelection, self->Delta );
}

void Editor_DrawSizeHighlight( Editor *self )
{
   RastPort *rport;
   Point size, location;

   rport    = RPort( InteractorWindow( (struct Interactor *)self ) );
   location = self->Location;
   size     = self->Size;

   SetDrMd ( rport, COMPLEMENT );
   RectFill( rport, location.x,          location.y,
                    location.x + size.x -1, location.y + size.y -1 );

}

#define DrawSizeHighlight(s) Editor_DrawSizeHighlight(s)
#define EraseSizeHighlight(s) DrawSizeHighlight(s)

void Editor_DrawLasso( Editor *self )
{
   Point location, handle;
   RastPort *rport;

   location = self->Location;
   handle   = self->Handle;

/*   printf("lasso=(%d,%d)-(%d,%d)\n",
 *     location.x, location.y, handle.x, handle.y );
 */

   rport = RPort( InteractorWindow( (struct Interactor *)self ) );
   SetDrMd( rport, COMPLEMENT );
   SetDrPt( rport, LassoPatterns[self->substate % LASSO_PATS] );
   Move   ( rport, handle.x, handle.y );
   Draw   ( rport, location.x, handle.y );
   Draw   ( rport, location.x, location.y );
   Draw   ( rport, handle.x,   location.y );
   Draw   ( rport, handle.x,   handle.y );

   SetDrPt( rport, 0xFFFF ); /* return to solid */
}

void Editor_FreeSelections( Editor *self )
{
   SelectedObject *si, *next_si;

   for( si = self->FirstSelection; si != NULL; si = next_si )
   {
      next_si = si->Next;
      Afree( si );
   }

   self->FirstSelection = NULL;
   self->nSelected      = 0;
}


void Editor_DeselectAll( Editor *self )
{
   pcgWindow *pw;

   /* printf("Deselecting Everything\n"); */
   pw = InteractorWindow( (struct Interactor *)self );

   so_DrawHighlights( RPort( pw ), self->FirstSelection, self->Delta );

   Editor_FreeSelections( self );
}

void Editor_DeselectPObject( Editor *self, GraphicObject *obj )
{
   SelectedObject *si;
   SelectedObject *prev_si = NULL;

   /* printf("Deselect object \"%s\"\n", obj->PObjectName ); */

   if( self->nSelected )
   {
      for( si = self->FirstSelection; si != NULL; si = si->Next )
      {
         if( si->PObject == obj )
         {
            /* Erase highlight. */
            so_DrawHighlight( RPort( InteractorWindow( (struct Interactor *)self ) ), si, self->Delta );

            if( prev_si )
                prev_si->Next = si->Next;
            else
                self->FirstSelection = si->Next;

            Afree( si );
            self->nSelected--;

            break;
         }
         prev_si = si;
      }
   }
}

SelectedObject*
Editor_SelectPObjectButDontHilite( Editor *self, GraphicObject *obj )
{
   SelectedObject *so      = NULL;

   if( so = so_SelectPObject( &self->FirstSelection, obj ) )
   {
      self->nSelected++;
   }
   return so;
}

void Editor_SelectPObject( Editor *self, GraphicObject *obj )
{
   SelectedObject *so      = NULL;

   if( so = Editor_SelectPObjectButDontHilite( self, obj ) )
   {
      /* printf("Drawing it...\n"); */
      so_DrawHighlight( RPort( InteractorWindow( (struct Interactor *)self ) ), so, self->Delta );
   }
}


BOOL Editor_InsideLasso( GraphicObject *self, Point min, Point max )
{
   Point location, size;

   location = Location( self );
   size     = Size( self );
           /* added cast to BOOL -- EDB */
   return( (BOOL) ( ( location.x >= min.x )  && ( location.y >= min.y ) &&
           ( location.x + size.x-1 <= max.x ) && ( location.y+size.y-1 <= max.y ) ) );
}


void Editor_SelectLasso( Editor *self )
{
   Point location, handle, min, max;
   GraphicObject *obj;
   pcgWindow *pw;

   /* printf("SelectLasso\n");  */

   location = self->Location;
   handle   = self->Handle;

   min.x = MIN( location.x, handle.x );
   min.y = MIN( location.y, handle.y );
   max.x = MAX( location.x, handle.x );
   max.y = MAX( location.y, handle.y );

   pw = InteractorWindow( (struct Interactor *)self );

   for( obj = self->Next; obj != NULL; obj = obj->Next )
   {
      if( Editor_InsideLasso( obj, min, max ) )
      {
         /* printf("Selecting object \"%s\"\n", obj->PObjectName);  */
         Editor_SelectPObject( self, obj );
      }
   }

   for( obj = pw->FirstGraphic; obj != NULL; obj = obj->Next )
   {
      if( Editor_InsideLasso( obj, min, max ) )
      {
         /* printf("Selecting object \"%s\"\n", obj->PObjectName);  */
         Editor_SelectPObject( self, obj );
      }
   }
}


Rectangle Editor_SelectionMBR( Editor *self )
{
   Rectangle      mbr;
   SelectedObject *si;
   Point          size, location;

   mbr.MinX = mbr.MinY =  32767;
   mbr.MaxX = mbr.MaxY = -32768;


   for( si = self->FirstSelection; si != NULL; si = si->Next )
   {
      location = Location( si->PObject );
      size     = Size( si->PObject );

      mbr.MinX = MIN( mbr.MinX, location.x );
      mbr.MinY = MIN( mbr.MinY, location.y );
      mbr.MaxX = MAX( mbr.MaxX, location.x + size.x-1 );
      mbr.MaxY = MAX( mbr.MaxY, location.y + size.y-1 );

   }

   return mbr;
}

void so_DepthArrange( pcgWindow *pw, SelectedObject *si, USHORT Depth_ID )
{
   if( si == NULL ) return;

   /*
   ** This recurses, so that the objects are placed in the front/or back
   ** in the order they were selected.  (A straight loop would reverse
   ** the order.)
   */

   so_DepthArrange( pw, si->Next, Depth_ID );

   switch( Depth_ID )
   {
      case EDIT_FRONT_ID:
         BuilderWindow_PObjectToFront( (struct BuilderWindow *)pw, si->PObject );
         break;

      case EDIT_BACK_ID:
         BuilderWindow_PObjectToBack( (struct BuilderWindow *)pw, si->PObject );
         break;
   }
}

void Editor_Align( Editor *self, USHORT Alignment_ID )
{
   Rectangle      mbr;
   SelectedObject *si;
   Point          location, size, mbr_center;

   mbr = Editor_SelectionMBR( self );
   mbr_center.x = ( mbr.MinX + mbr.MaxX ) / 2;
   mbr_center.y = ( mbr.MinY + mbr.MaxY ) / 2;


   for( si = self->FirstSelection; si != NULL; si = si->Next )
   {
      location   = Location( si->PObject );
      size       = Size( si->PObject );


      switch( Alignment_ID )
      {
         case ALIGN_LEFT_ID:
            location.x = mbr.MinX;
            break;

         case ALIGN_RIGHT_ID:
            location.x = mbr.MaxX - size.x + 1;
            break;

         case ALIGN_CENTERX_ID:
            location.x = mbr_center.x - size.x/2;
            break;

         case ALIGN_TOP_ID:
            location.y = mbr.MinY;
            break;

         case ALIGN_BOTTOM_ID:
            location.y = mbr.MaxY - size.y + 1;
            break;

         case ALIGN_CENTERY_ID:
            location.y = mbr_center.y - size.y/2;
            break;

      }

      SetLocation( si->PObject, location.x, location.y );

   }
   Project_Modify();
}

void Editor_GotoWaitState( Editor *self )
{
   pcgWindow *window;

   /* printf("going to wait state.\n");  */

   window = InteractorWindow( (struct Interactor *)self );

   self->state = EDITOR_WAITING;
   /* printf("Turning mousemove OFF\n");  */
   SetIDCMPFlags( window, self->IDCMPBuf );
   ReportMouseMove( window->Window, FALSE );
   ReportRightMouseButton( window->Window, FALSE );
   ClearPointer( window->Window );

   self->Target = NULL;
}

#define GotoWaitState(s) Editor_GotoWaitState(s)

void Editor_StartDragging( Editor *self, Point mouse )
{
   pcgWindow *window;
   ULONG newflags;
   Point loc;

   window = InteractorWindow( (struct Interactor *)self );
   loc = Location( self->Target );

   self->IDCMPBuf   = IDCMPFlags( (struct Interactor *)window );
   self->state      = EDITOR_DRAGGING;
   self->Delta.x    = 0;
   self->Delta.y    = 0;
   self->Handle.x   = mouse.x - loc.x; /* Offset between mouse pointer and */
   self->Handle.y   = mouse.y - loc.y; /* object being dragged.            */
   self->Location   = mouse;

   newflags = self->IDCMPBuf | MOUSEMOVE;
   SetIDCMPFlags( window, newflags );
   ReportMouseMove( window->Window, (ULONG) TRUE );
   ReportRightMouseButton( window->Window, TRUE );
   EditBox_SetPointer( window->Window, eb_Hand );

   QueueFunc( &DeferredResponses,
               (ptr2func)Editor_BeSelfish, self );

}

void Editor_StartLassoing( Editor *self, Point mouse )
{
   pcgWindow *window;
   ULONG newflags;

   window = InteractorWindow( (struct Interactor *)self );
   /* printf("starting to lasso on window %s\n", Title(window)); */


   self->IDCMPBuf   = IDCMPFlags( (struct Interactor *)window );
   self->state      = EDITOR_LASSOING;
   self->substate   = 0;
   self->Delta.x    = 0;
   self->Delta.y    = 0;
   self->Handle     = mouse;
   self->Location   = mouse;

   /* printf("Turning mousemove ON.\n"); */
   newflags = self->IDCMPBuf | MOUSEMOVE | INTUITICKS;
   SetIDCMPFlags( window, newflags );
   ReportMouseMove( window->Window, TRUE );

   Editor_DrawLasso(self);
}


void Editor_StartSizing( Editor *self, Point mouse, HitClass hitclass )
{
   pcgWindow *window;
   Point size, location;
   window = InteractorWindow( (struct Interactor *)self );


   self->IDCMPBuf   = IDCMPFlags( (struct Interactor *)window );
   self->state      = EDITOR_SIZING;
   self->substate   = hitclass;
   self->Delta.x    = 0;
   self->Delta.y    = 0;
   size             = Size( self->Target );
   location         = Location( self->Target );
   self->Location   = location;
   self->Size       = size;

   if( hitclass & LEFTHIT )
      self->Handle.x = mouse.x - location.x;
   else
      if( hitclass & RIGHTHIT )
         self->Handle.x = mouse.x - ( location.x + size.x-1 );
      else
         self->Handle.x = 0;

   if( hitclass & TOPHIT )
      self->Handle.y = mouse.y - location.y;
   else
      if( hitclass & BOTTOMHIT )
         self->Handle.y = mouse.y - ( location.y + size.y -1 );
      else
         self->Handle.y = 0;

   ModifyIDCMP( window->Window, (ULONG) self->IDCMPBuf | MOUSEMOVE );
   ReportMouseMove( window->Window, TRUE );
   ReportRightMouseButton( window->Window, TRUE );

   switch( hitclass )
   {
      case LEFTHIT:
         EditBox_SetPointer( window->Window, eb_LeftArrow );
         break;

      case LEFTTOPHIT:
         EditBox_SetPointer( window->Window, eb_LeftTopArrow );
         break;

      case TOPHIT:
         EditBox_SetPointer( window->Window, eb_TopArrow );
         break;

      case RIGHTTOPHIT:
         EditBox_SetPointer( window->Window, eb_RightTopArrow );
         break;

      case RIGHTHIT:
         EditBox_SetPointer( window->Window, eb_RightArrow );
         break;

      case RIGHTBOTTOMHIT:
         EditBox_SetPointer( window->Window, eb_RightBottomArrow );
         break;

      case BOTTOMHIT:
         EditBox_SetPointer( window->Window, eb_BottomArrow );
         break;

      case LEFTBOTTOMHIT:
         EditBox_SetPointer( window->Window, eb_LeftBottomArrow );
         break;
   }
}


USHORT Editor_XSelectRespond( Editor *self, pcgWindow *pw, Point mouse )
{
   HitInfo hitinfo;

   /* printf("extended select\n"); */
   hitinfo = FindHitSelection( self->FirstSelection, mouse );

#if 0
   printf("Hitclass=%d", hitinfo.hitclass );
   if (hitinfo.PObject)
      printf(", PObject=\"%s\"\n", hitinfo.PObject->PObjectName );
   else
      printf("\n");
#endif

   if( hitinfo.hitclass )
   {
      Editor_DeselectPObject( self, hitinfo.PObject );
   }
   else
   {
      /* printf("didn't hit anything selected.\n");  */
      hitinfo = FindHitPObject( self->Next, mouse );
      if( hitinfo.hitclass == NONE )
         hitinfo = FindHitPObject( pw->FirstGraphic, mouse );

#if 0
      printf("XSelect: Hitclass=%d", hitinfo.hitclass );

      if (hitinfo.PObject)
         printf(", PObject=\"%s\"\n", hitinfo.PObject->PObjectName );
      else
         printf("\n");
#endif

      if( hitinfo.hitclass )
      {
         Editor_SelectPObject( self, hitinfo.PObject );

         self->Target = hitinfo.PObject;

         if ( ( self->nSelected  == 1 ) &&
              ( hitinfo.hitclass != CENTERHIT ) )
         {
            /* printf("Start sizing...\n");  */
            Editor_StartSizing( self, mouse, hitinfo.hitclass );
         }
         else
         {
            /* printf("start dragging...\n"); */
            Editor_StartDragging( self, mouse );
         }
      }
      else
      {
         Editor_StartLassoing( self, mouse );
      }
   }
   return RESPONDED | CHANGED_STATE;
}


USHORT Editor_SelectRespond( Editor *self, pcgWindow *pw, Point mouse )
{
   HitInfo hitinfo;

   /* printf("normal select\n" ); */

   hitinfo = FindHitSelection( self->FirstSelection, mouse );
   if( hitinfo.hitclass )
   {
      if( self->nSelected > 1 )      /* if >1, can't do sizing. */
      {
         self->Target = hitinfo.PObject;
         Editor_StartDragging( self, mouse );
      }
      else                  /* only 1 object -> can do sizing. */
      {
         self->Target = hitinfo.PObject;
         if( hitinfo.hitclass == CENTERHIT )
         {
            Editor_StartDragging( self, mouse );
         }
         else
         {
            Editor_StartSizing( self, mouse, hitinfo.hitclass );
         }
      }
   }
   else                       /* didn't hit a selected object. */
   {
      /* printf("didn't hit any selected.\n"); */
      Editor_DeselectAll( self );
      hitinfo = FindHitPObject( self->Next, mouse );
      if( hitinfo.hitclass == NONE )
         hitinfo = FindHitPObject( pw->FirstGraphic, mouse );

      if( hitinfo.hitclass )
      {
         Editor_SelectPObject( self, hitinfo.PObject );
         self->Target = hitinfo.PObject;
         if( hitinfo.hitclass == CENTERHIT )
            Editor_StartDragging( self, mouse );
         else
            Editor_StartSizing( self, mouse, hitinfo.hitclass );
      }
      else  /* didn't hit anything at all. */
      {
         /* printf("really didnt hit anything.\n"); */
         Editor_DeselectAll( self );
         Editor_StartLassoing( self, mouse );
      }
   }

   return RESPONDED | CHANGED_STATE;
}


USHORT Editor_WaitingRespond( Editor *self, IntuiMessage *event )
{
   USHORT      response = 0;
   Point       mouse;
   pcgWindow  *pw;
   static ULONG glsecs = 0L, glmics = 0L; /* for gadget doubleclick */

   /* printf("Waiting respond.\n");  */
   switch( event->Class )
   {
      case MOUSEBUTTONS:
         switch( event->Code )
         {
            case SELECTDOWN:
               pw = InteractorWindow( (struct Interactor *)self );
               if( event->IDCMPWindow != pw->Window ) break;

               mouse.x = event->MouseX;
               mouse.y = event->MouseY;

               if( event->Qualifier & EITHER_SHIFT ) /* Extended selection */
                  response = Editor_XSelectRespond( self, pw, mouse );
               else /* Not extended selection. */
                  {
                  if( DoubleClick( glsecs, glmics, event->Seconds, event->Micros ) )
                    {
                      /* EDB -- DoubleClick in Editor Window */
                      /* Edit Object Properties as if menu item was chosen */

                   /* EDB -- Small BUG left to fix here... */
                   /*
                    * Should check hit coordinates are inside of
                    * self->FirstSelection->PObject before popping up the
                    * object properties editor window
                    */

                     if( self->nSelected == 1 )
                       {
                         Editor_PropEdit( self, self->FirstSelection->PObject );
                         response = RESPONDED | CHANGED_STATE;
                       }
                     else /* no previously selected object */
                       {
                          /* allow normal selection */
                       response = Editor_SelectRespond( self, pw, mouse );
                       }

                    }
                  else  /* not a double click selection */
                    {
                      glsecs = event->Seconds;
                      glmics = event->Micros;
                      response = Editor_SelectRespond( self, pw, mouse );
                    }

                  }
               break;
         }
   }


   return response;
}


USHORT Editor_LassoingRespond( Editor *self, IntuiMessage *event )
{
   USHORT response = 0;

  /* printf("lasso respond.\n");  */

   switch( event->Class )
   {
      case INTUITICKS:
      case MOUSEMOVE:
         /* printf("Lasso mousemove.\n");  */
         EraseLasso( self );
         self->Handle.x = event->MouseX;
         self->Handle.y = event->MouseY;
         self->substate ++;
         DrawLasso( self );
         response = RESPONDED | CHANGED_STATE;
         break;

      case MOUSEBUTTONS:
         switch( event->Code )
         {
            case SELECTUP:
               /* printf("Lasso selectup\n"); */
               EraseLasso( self );
               self->Handle.x = event->MouseX;
               self->Handle.y = event->MouseY;
               DrawLasso( self );

               EraseLasso( self );
               Editor_SelectLasso( self );
               GotoWaitState( self );
               response = RESPONDED | CHANGED_STATE;
               break;

            case MENUDOWN: /* cancel. */
               /* printf("Lasso menudown\n"); */
               EraseLasso( self );
               GotoWaitState( self );
               response = RESPONDED | CHANGED_STATE;
               break;
         }
   }

   return response;
}

void Editor_MovePObjects( Editor *self )
{
   SelectedObject *si;
   GraphicObject  *obj;
   Point location;
   /* EDB */
   /* Additions for snapping objects inside window borders */
   Point window_size, object_size;
   Window *iwindow;
   short leftborder, rightborder, topborder, bottomborder;

   iwindow = iWindow( self->IaWindow );
   leftborder = iwindow->BorderLeft;
   rightborder = iwindow->BorderRight;
   topborder = iwindow->BorderTop;
   bottomborder = iwindow->BorderBottom;

   window_size = Size( (struct GraphicObject *)self->IaWindow );
   for( si = self->FirstSelection; si != NULL; si = si->Next )
   {
      obj = si->PObject;

      location = Location( obj );
      object_size = Size( obj );

      if( ( (location.x + self->Delta.x) > leftborder ) &&
          ( (location.x + self->Delta.x + object_size.x) < (window_size.x - rightborder) ) )
        {
         /* if object is within window borders */
        location.x += self->Delta.x;
        }

      else /* snap to inside left or right window border */
        {
        if( (location.x + self->Delta.x) > leftborder )
           /* right of left border, snap to right border */
           location.x = (window_size.x - rightborder) - object_size.x;
        else /* snap to left border */
           location.x = leftborder;
        }

      if( ( (location.y + self->Delta.y) > topborder ) &&
          ( (location.y + self->Delta.y + object_size.y) < (window_size.y - bottomborder) ) )

         /* if object is within window borders */
        location.y += self->Delta.y;

      else /* snap to inside top or bottom window border */
        {
        if( (location.y + self->Delta.y) > topborder )
           /* below top border, snap to bottom */
           location.y = window_size.y - bottomborder - object_size.y;
        else /* snap to top border */
           location.y = topborder;
        }

      SetLocation( obj, location.x, location.y );
   }
   Project_Modify();
}

#define DrawHighlights(s) Editor_DrawHighlights(s)
#define EraseHighlights(s) DrawHighlights(s)

USHORT Editor_DraggingRespond( Editor *self, IntuiMessage *event )
{
   USHORT response = 0;
   Point new_loc, old_loc;

   /* printf("dragging respond.\n"); */

   switch( event->Class )
   {
      case MOUSEMOVE:
         /* printf("Lasso mousemove.\n"); */
         EraseHighlights( self );

         old_loc   = Location( self->Target );
         new_loc.x = event->MouseX - self->Handle.x;
         new_loc.y = event->MouseY - self->Handle.y;

         if (self->gridinfo)
            new_loc = GridSnap( new_loc, self->gridinfo->location );

         self->Delta.x = new_loc.x - old_loc.x;
         self->Delta.y = new_loc.y - old_loc.y;

         DrawHighlights( self );
         response = RESPONDED | CHANGED_STATE;
         break;

      case MOUSEBUTTONS:
         switch( event->Code )
         {
            case SELECTUP:
               /* printf("Lasso selectup\n"); */
               if( self->Delta.x != 0 || self->Delta.y != 0 )
               {  /* Only do this if the objects were actually moved. */
                  EraseHighlights( self );
                  Editor_MovePObjects( self );

                  self->Delta.x = 0;
                  self->Delta.y = 0;

                  Render( (struct GraphicObject *)InteractorWindow( (struct Interactor *)self ), NULL ); /* Redraw. */
               }
               GotoWaitState( self );
               response = RESPONDED | CHANGED_STATE;
               break;

            case MENUDOWN: /* cancel. */
               /* printf("Lasso menudown\n"); */
               EraseHighlights( self );
               self->Delta.x = 0;
               self->Delta.y = 0;
               DrawHighlights(self);
               GotoWaitState(self);
               response = RESPONDED | CHANGED_STATE;
               break;
         }
   }

   return response;
}

void Editor_ChangeSize( Editor *self, Point mouse )
{
   tPoint new_size, new_location;
   USHORT size_mode;

   /* EDB */
   /* Additions for snapping objects inside window borders */
   Point window_size;
   Window *iwindow;
   short leftborder, rightborder, topborder, bottomborder;
   BOOL snapleft = FALSE;
   BOOL snaptop = FALSE;

   iwindow = iWindow( self->IaWindow );
   leftborder = iwindow->BorderLeft;
   rightborder = iwindow->BorderRight;
   topborder = iwindow->BorderTop;
   bottomborder = iwindow->BorderBottom;

   window_size = Size( (struct GraphicObject *)self->IaWindow );

   size_mode    = self->substate;
   new_size     = self->Size;
   new_location = self->Location;

   mouse.x -= self->Handle.x;
   mouse.y -= self->Handle.y;

   if( self->gridinfo )
      mouse = GridSnap( mouse, self->gridinfo->size );


   if( size_mode & LEFTHIT )
     {
      /* check sizing against left window border */
        if( ( self->Location.x + mouse.x ) <= leftborder )
          {
           snapleft = TRUE;
           new_size.x = ( self->Size.x + (self->Location.x - leftborder) );
          }
        else
          new_size.x = ( self->Location.x + self->Size.x-1 ) - mouse.x;
     }
   else if( size_mode & RIGHTHIT )
     {
      /* check sizing against right window border */
         if( ( self->Location.x + (mouse.x - self->Location.x) )
             > (window_size.x - rightborder) )
           {
            new_size.x = window_size.x - rightborder - self->Location.x;
           }
         else
            new_size.x = mouse.x  - self->Location.x;
     }

   if( size_mode & TOPHIT )
     {
      /* check sizing against top window border */
      if( ( self->Location.y + mouse.y ) <= topborder )
        {
        snaptop = TRUE;
        new_size.y = ( self->Size.y + (self->Location.y - topborder) );
        }
      else
        new_size.y = (self->Location.y + self->Size.y-1) - mouse.y;
     }
   else if( size_mode & BOTTOMHIT )
     {
      /* check sizing against bottom window border */
      if( ( self->Location.y + (mouse.y - self->Location.y) )
          > (window_size.y - bottomborder) )
        {
          new_size.y = window_size.y - bottomborder - self->Location.y;
        }
      else
        new_size.y = mouse.y  - self->Location.y;
     }

   new_size = AskSize( self->Target, new_size.x, new_size.y );

   if( size_mode & LEFTHIT )
     {
      /* check location against left window border */
      if( !snapleft )
          new_location.x = (self->Location.x + self->Size.x) - new_size.x;
      else
          new_location.x = leftborder;
     }

   if( size_mode & TOPHIT )
     {
      /* check location against top window border */
      if( !snaptop )
           new_location.y = (self->Location.y + self->Size.y) - new_size.y;
      else
           new_location.y = topborder;
     }

   self->Size     = new_size;
   self->Location = new_location;
}

#define ChangeSize(s,m) Editor_ChangeSize(s,m)

BOOL Editor_ApplyEdits( Editor *self )
{
   tPoint size, location;

   /* printf("Change size for \"%s\"\n", self->Target->PObjectName );  */
   location = Location( self->Target );
   size     = Size( self->Target );

   if ( (self->Size.x     != size.x)
      | (self->Size.y     != size.y)
      | (self->Location.x != location.x)
      | (self->Location.y != location.y))
   {

#if 0
      printf("Changing from size (%d,%d) (%dx%d)\n",
         location.x, location.y, size.x, size.y );
      printf("   to (%d,%d) (%dx%d)\n",
         self->Location.x, self->Location.y,
         self->Size.x, self->Size.y );
#endif

      SetSize    ( self->Target, self->Size.x,     self->Size.y );
      SetLocation( self->Target, self->Location.x, self->Location.y );

      Project_Modify();
      return TRUE;
   }
   else return FALSE; /* no change. */
}

#define ApplyEdits(s) Editor_ApplyEdits(s)

USHORT Editor_SizingRespond( Editor *self, IntuiMessage *event )
{
   tPoint         mouse;
   USHORT         response = 0;

   switch( event->Class )
   {
      case MOUSEMOVE:
         mouse.x = event->MouseX;
         mouse.y = event->MouseY;
         EraseSizeHighlight( self );
         ChangeSize( self, mouse );
         DrawSizeHighlight( self );
         response = RESPONDED | CHANGED_STATE;
     /*   printf("size = (%dx%d)\n", self->Size.x, self->Size.y );  */
         break;

      case MOUSEBUTTONS:
         switch( event->Code )
         {
            case SELECTUP:
           /*    printf("size = (%dx%d)\n", self->Size.x, self->Size.y ); */
               EraseSizeHighlight( self );
               if( ApplyEdits( self ) )
               {
                  Render( (struct GraphicObject *)InteractorWindow( (struct Interactor *)self ), NULL );
               }
               else
                  DrawHighlights( self );

               GotoWaitState( self );
               response = RESPONDED | CHANGED_STATE;
               break;


            case MENUDOWN:
               EraseSizeHighlight( self );
               GotoWaitState( self );
               DrawHighlights( self );
               response = RESPONDED | CHANGED_STATE;
               break;
         }
         break;
   }

   return response;
}

USHORT Editor_ReadyMenus( Editor *self, Window *w )
{
   /* Ready the menus for viewing. */
   if( self->nSelected )
   {
      OnMenu( w, mn_Cut );
      OnMenu( w, mn_Copy );
      OnMenu( w, mn_Front );
      OnMenu( w, mn_Back );

#if 0
      OnMenu( w, mn_Group );
      OnMenu( w, mn_Ungroup );
#endif

      if( self->nSelected > 1 )
      {
         OffMenu( w, mn_Props );
         OnMenu ( w, mn_Alignment | (NOITEM<<11) );
      }
      else
      {
         OnMenu ( w, mn_Props );
         OffMenu( w, mn_Alignment | (NOITEM<<11) );
      }
   }
   else
   {
      OffMenu( w, mn_Cut );
      OffMenu( w, mn_Copy );
      OffMenu( w, mn_Props );
      OffMenu( w, mn_Front );
      OffMenu( w, mn_Back );

#if 0
      OffMenu( w, mn_Group );
      OffMenu( w, mn_Ungroup );
#endif

      OffMenu( w, mn_Alignment | (NOITEM<<11) );
   }

   if( self->pasteBuffer->length )
   {
      OnMenu( w, mn_Paste );
   }
   else
   {
      OffMenu( w, mn_Paste );
   }

   return RESPONDED | CHANGED_STATE;
}

void Editor_Cut( Editor *self )
{
   /* NOTE: This is a true Cut, not a Delete. */

   SelectedObject *so, *next_so;
   GraphicObject  *obj;
   pcgWindow      *window;

   window = InteractorWindow( (struct Interactor *)self );
   SetWaitPointer( iWindow( window ) );

/* Empty out the old paste buffer */
   setLengthOfArray( self->pasteBuffer, 0 );

   for( so = self->FirstSelection; so != NULL; so = next_so )
   {
      next_so = so->Next;

      obj = so->PObject;

      RemoveWindowPObject( window, obj );
      appendToArray( self->pasteBuffer, obj );
      Afree( so );
      Project_Modify();
   }
   self->FirstSelection = NULL;
   self->nSelected      = 0;
   Render( (struct GraphicObject *)window, NULL );

   ClearPointer( iWindow( window ) );
}

#define COPY_OFFSET 8

void Editor_Copy( Editor *self )
{
   SelectedObject *so, *next_so;
   GraphicObject  *obj, *copied_obj;
   pcgWindow      *window;
   Point           location;

   window = InteractorWindow( (struct Interactor *)self );
   SetWaitPointer( iWindow( window ) );

/* Empty out the old paste buffer */
   setLengthOfArray( self->pasteBuffer, 0 );

   for( so = self->FirstSelection; so != NULL; so = next_so )
   {
      next_so = so->Next;
      obj = so->PObject;

      if( copied_obj = New( (struct PObject *)obj ) )
      {
         location = Location( obj );
         /* offset the location of the copied object. */
         SetLocation( copied_obj, location.x + COPY_OFFSET, location.y + COPY_OFFSET );
         appendToArray( self->pasteBuffer, copied_obj );
      }
   }
   ClearPointer( iWindow( window ) );
}

void Editor_Paste( Editor *self )
{
   GraphicObject  *obj;
   pcgWindow      *window;
   Window         *iwindow;
   int i;

   window = InteractorWindow( (struct Interactor *)self );
   iwindow = iWindow( window );

   SetWaitPointer( iwindow );

   Editor_DeselectAll( self );

   for( i = 0; i < self->pasteBuffer->length; i++ )
   {
      obj = (GraphicObject*) self->pasteBuffer->vec[i];
      self->pasteBuffer->vec[i] = NULL;
      BuilderWindow_AddPObjectButDontRender( window, obj );
      Project_Modify();
      Editor_SelectPObjectButDontHilite( self, obj );
   }
   Render( (struct GraphicObject *)window, NULL );

/* Empty out the old paste buffer */
   setLengthOfArray( self->pasteBuffer, 0 );

/* Put fresh copies into the buffer */
   Editor_Copy( self );
   ClearPointer( iwindow );

}




USHORT Editor_ParseMenuEvent( Editor  *self,
                              USHORT   MenuNumber,
                              Window  *window )
{
   pcg_MenuItem *item;
   USHORT        response = 0;

   if( MenuNumber == MENUNULL ) return response;

   item = (pcg_MenuItem *)
      ItemAddress( window->MenuStrip, MenuNumber );

   switch( item->ItemID )
   {
      case EDIT_CUT_ID:
         Editor_Cut( self );
         response = RESPONDED | CHANGED_STATE;
         break;

      case EDIT_COPY_ID:
         Editor_Copy( self );
         response = RESPONDED | CHANGED_STATE;
         break;

      case EDIT_PASTE_ID:
         Editor_Paste( self );
         response = RESPONDED | CHANGED_STATE;
         break;

      case EDIT_PROPS_ID:
         if( self->nSelected == 1 )
            Editor_PropEdit( self, self->FirstSelection->PObject );
         response = RESPONDED | CHANGED_STATE;
         break;

      case EDIT_WINDOW_ID:
         Editor_PropEdit( self, (struct GraphicObject *)InteractorWindow( (struct Interactor *)self ) );
         response = RESPONDED | CHANGED_STATE;
         break;

      case EDIT_FRONT_ID:
      case EDIT_BACK_ID:
         so_DepthArrange( InteractorWindow( (struct Interactor *)self ),
                          self->FirstSelection,
                          item->ItemID );
         response = RESPONDED | CHANGED_STATE;
         break;

      case ALIGN_LEFT_ID:
      case ALIGN_RIGHT_ID:
      case ALIGN_CENTERX_ID:
      case ALIGN_TOP_ID:
      case ALIGN_BOTTOM_ID:
      case ALIGN_CENTERY_ID:
         Editor_Align( self, item->ItemID );
         Render( (struct GraphicObject *)InteractorWindow( (struct Interactor *)self ), NULL );
         response = RESPONDED | CHANGED_STATE;
         break;
   }

   return (USHORT) response;
   /*  return (USHORT) response | Editor_ParseMenuEvent( self, item->NextSelect, window ); */
}


USHORT Editor_Respond( Editor *self, IntuiMessage *event )
{
   USHORT response = 0;
   pcgWindow *pw;

/* printf("Editor_Respond\n"); */

   switch (event->Class)
   {
      case MENUPICK:
         response = Editor_ParseMenuEvent( self, event->Code, event->IDCMPWindow );
         break;

      case MENUVERIFY:
         response = Editor_ReadyMenus( self, event->IDCMPWindow );
         break;

      default:
         pw = InteractorWindow( (struct Interactor *)self );

         if( event->IDCMPWindow == pw->Window )
         {
            switch( self->state )
            {
               case EDITOR_WAITING:
                  response = Editor_WaitingRespond( self, event );
                  break;

               case EDITOR_LASSOING:
                  response = Editor_LassoingRespond( self, event );
                  break;

               case EDITOR_SIZING:
                  response = Editor_SizingRespond( self, event );
                  break;

               case EDITOR_DRAGGING:
                  response = Editor_DraggingRespond( self, event );
                  break;
            }
        }
   }

   return response;
}

void Editor_Refresh( Editor *self )
{
   pcgWindow *w;
   RastPort *rport;

   w     = InteractorWindow( (struct Interactor *)self );
   rport = RPort( w );

   Render( (struct GraphicObject *)self, rport );
}


void Editor_Render( Editor *self, RastPort *RPort )
{
   if( self->state == EDITOR_SIZING )
      DrawSizeHighlight( self );
   else
      so_DrawHighlights( RPort, self->FirstSelection, self->Delta );
}

void Editor_Init( Editor *self, pcg_3DPens pens, GridInfo *gridinfo )
{
   Interactor_Init( (struct Interactor *)self );
   self->isa            = EditorClass();
   self->state          = EDITOR_WAITING;
   self->substate       = 0;
   self->FirstSelection = NULL;
   self->nSelected      = 0;
   self->Delta.x        = 0;
   self->Delta.y        = 0;
   self->Target         = NULL;
   self->gridinfo       = gridinfo;
   self->pens           = pens;

   self->pasteBuffer    = newArray( 0 );

   setArrayDestructor( self->pasteBuffer, (ArrayDtor)Free );

   SetObjectName( (struct PObject *)self, "*Editor*" );
}


ULONG Editor_IDCMPFlags( Editor *self )
{
   ULONG flags = MOUSEBUTTONS | REFRESHWINDOW | MENUPICK;

   if( self->state )
      flags |= MOUSEMOVE;

   return flags;
}


tPoint Editor_AskSize( Editor *self, PIXELS Width, PIXELS Height )
{
   Point dummy;

   if( self->Target )
      return AskSize( self->Target, Width, Height );

   dummy.x = Width;
   dummy.y = Height;
   return dummy;
}


tPoint Editor_SetSize( Editor *self, PIXELS Width, PIXELS Height )
{
   tPoint size;

   self->Size = size = AskSize( (struct GraphicObject *)self, Width, Height );

   return size;
}


USHORT Editor_SizeFlags( Editor *self )
{
   return 0;
}


void Editor_CleanUp( Editor *self )
{
   Editor_FreeSelections( self );
   freeArray( self->pasteBuffer ); /* Array destructor will empty itself as needed. */
}



long WriteEditorChunk( GraphicObject *bgad, FILE *file )
{
   return 0;
}

BOOL Editor_elaborated = FALSE;
struct BuilderMethods Editor_bm;

struct InteractorClass Editor_Class;

void EditorClass_Init( struct InteractorClass *class )
{
   InteractorClass_Init( class );
   class->isa                 = InteractorClass();
   class->ClassName           = "Editor";
   class->CleanUp             = (void(*)(PObject *))Editor_CleanUp;
   class->AskSize             = (Point(*)(GraphicObject *, PIXELS, PIXELS))Editor_AskSize;
   class->SetSize             = (Point(*)(GraphicObject *, PIXELS, PIXELS))Editor_SetSize;
   class->SizeFlags           = (UWORD(*)(GraphicObject *))Editor_SizeFlags;
   class->Render              = (void(*)(GraphicObject *, RastPort *))Editor_Render;
   class->FirstGadget         = NULL;
   class->nGadgets            = NULL;
   class->IDCMPFlags          = (ULONG(*)(Interactor *))Editor_IDCMPFlags;
   class->ClaimEvent          = NULL;
   class->Respond             = (USHORT(*)(Interactor *, IntuiMessage *))Editor_Respond;
   class->Refresh             = (void(*)(Interactor *))Editor_Refresh;
   class->EnableIactor        = NULL;
   class->isEnabled           = NULL;
   class->Activate            = NULL;
   class->isActive            = NULL;
   class->SetTitle            = NULL;
   class->Title               = NULL;


   class->BuilderMethods = &Editor_bm;
   Editor_bm.New       = NULL;
   Editor_bm.WriteCode = NULL;
   Editor_bm.WriteIFF  = WriteEditorChunk;
   Editor_bm.ReadIFF   = NULL;

}


struct InteractorClass *EditorClass( void )
{
   if( ! Editor_elaborated )
   {
      EditorClass_Init( &Editor_Class );
      Editor_elaborated = TRUE;
   }

   return &Editor_Class;
}

extern struct MsgPort *shared_port;

/* note: Changed to fit same algorithm as ToolWindow_BeSelfish -- EDB */
/*       to remove the missed select up message bug.                  */

void Editor_BeSelfish( Editor *self )
{
   struct IntuiMessage *message, local_msg;
   BOOL got_a_msg;

   while( self->state != EDITOR_WAITING )
   {
      got_a_msg = FALSE;

      for( ;; ) /* Strip all events from message queue. */
      {
         message = (struct IntuiMessage *)GetMsg( shared_port );

         if( message )
         {
            local_msg = *message;
            ReplyMsg( (struct Message *)message );
            Editor_Respond( self, &local_msg );
            got_a_msg = TRUE;

         }
         else /* no message */
         {
            if( got_a_msg ) /* We responded to all of them */
               break;
            else           /* no message to respond to yet */
               WaitPort( shared_port );
         }
      }
   }

}
